@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: cu_algorithm.c 4168 2009-06-11 16:44:03Z pramsey $
+ * $Id: cu_algorithm.c 5181 2010-02-01 17:35:55Z pramsey $
  *
  * PostGIS - Spatial Types for PostgreSQL
  * http://postgis.refractions.net
@@ -30,6 +30,7 @@ CU_pSuite register_cg_suite(void)
 	    (NULL == CU_add_test(pSuite, "test_lw_segment_intersects()", test_lw_segment_intersects)) ||
 	    (NULL == CU_add_test(pSuite, "test_lwline_crossing_short_lines()", test_lwline_crossing_short_lines)) ||
 	    (NULL == CU_add_test(pSuite, "test_lwline_crossing_long_lines()", test_lwline_crossing_long_lines)) ||
+	    (NULL == CU_add_test(pSuite, "test_lwline_crossing_bugs()", test_lwline_crossing_bugs)) ||
 	    (NULL == CU_add_test(pSuite, "test_lwpoint_set_ordinate()", test_lwpoint_set_ordinate)) ||
 	    (NULL == CU_add_test(pSuite, "test_lwpoint_get_ordinate()", test_lwpoint_get_ordinate)) ||
 	    (NULL == CU_add_test(pSuite, "test_lwpoint_interpolate()", test_lwpoint_interpolate)) ||
@@ -51,20 +52,12 @@ CU_pSuite register_cg_suite(void)
 /*
 ** Global variables used by tests below
 */
-POINT2D *p1 = NULL;
-POINT2D *p2 = NULL;
-POINT2D *q1 = NULL;
-POINT2D *q2 = NULL;
-POINT4D *p = NULL;
-POINT4D *q = NULL;
+
 /* Two-point objects */
 POINTARRAY *pa21 = NULL;
 POINTARRAY *pa22 = NULL;
 LWLINE *l21 = NULL;
 LWLINE *l22 = NULL;
-/* Five-point objects */
-LWLINE *l51 = NULL;
-LWLINE *l52 = NULL;
 /* Parsing support */
 LWGEOM_PARSER_RESULT parse_result;
 
@@ -75,12 +68,6 @@ LWGEOM_PARSER_RESULT parse_result;
 */
 int init_cg_suite(void)
 {
-	p = lwalloc(sizeof(POINT4D));
-	q = lwalloc(sizeof(POINT4D));
-	p1 = lwalloc(sizeof(POINT2D));
-	p2 = lwalloc(sizeof(POINT2D));
-	q1 = lwalloc(sizeof(POINT2D));
-	q2 = lwalloc(sizeof(POINT2D));
 	pa21 = ptarray_construct(0, 0, 2);
 	pa22 = ptarray_construct(0, 0, 2);
 	l21 = lwline_construct(-1, NULL, pa21);
@@ -95,11 +82,6 @@ int init_cg_suite(void)
 */
 int clean_cg_suite(void)
 {
-	if ( p ) lwfree(p);
-	if ( p1 )lwfree(p1);
-	if ( p2 ) lwfree(p2);
-	if ( q1 ) lwfree(q1);
-	if ( q2 ) lwfree(q2);
 	if ( l21 ) lwline_free(l21);
 	if ( l22 ) lwline_free(l22);
 	return 0;
@@ -111,36 +93,33 @@ int clean_cg_suite(void)
 void test_lw_segment_side(void)
 {
 	double rv = 0.0;
-	POINT2D *q = NULL;
-	q = lwalloc(sizeof(POINT2D));
+	POINT2D p1, p2, q;
 
 	/* Vertical line at x=0 */
-	p1->x = 0.0;
-	p1->y = 0.0;
-	p2->x = 0.0;
-	p2->y = 1.0;
+	p1.x = 0.0;
+	p1.y = 0.0;
+	p2.x = 0.0;
+	p2.y = 1.0;
 
 	/* On the left */
-	q->x = -2.0;
-	q->y = 1.5;
-	rv = lw_segment_side(p1, p2, q);
+	q.x = -2.0;
+	q.y = 1.5;
+	rv = lw_segment_side(&p1, &p2, &q);
 	//printf("left %g\n",rv);
 	CU_ASSERT(rv < 0.0);
 
 	/* On the right */
-	q->x = 2.0;
-	rv = lw_segment_side(p1, p2, q);
+	q.x = 2.0;
+	rv = lw_segment_side(&p1, &p2, &q);
 	//printf("right %g\n",rv);
 	CU_ASSERT(rv > 0.0);
 
 	/* On the line */
-	q->x = 0.0;
-	rv = lw_segment_side(p1, p2, q);
+	q.x = 0.0;
+	rv = lw_segment_side(&p1, &p2, &q);
 	//printf("on line %g\n",rv);
 	CU_ASSERT_EQUAL(rv, 0.0);
 
-	lwfree(q);
-
 }
 
 /*
@@ -149,189 +128,203 @@ void test_lw_segment_side(void)
 void test_lw_segment_intersects(void)
 {
 
+#define setpoint(p, x1, y1) {(p).x = (x1); (p).y = (y1);}
+
+	POINT2D p1, p2, q1, q2;
+
 	/* P: Vertical line at x=0 */
-	p1->x = 0.0;
-	p1->y = 0.0;
-	p2->x = 0.0;
-	p2->y = 1.0;
+	setpoint(p1, 0.0, 0.0);
+	p1.x = 0.0;
+	p1.y = 0.0;
+	p2.x = 0.0;
+	p2.y = 1.0;
 
 	/* Q: Horizontal line crossing left to right */
-	q1->x = -0.5;
-	q1->y = 0.5;
-	q2->x = 0.5;
-	q2->y = 0.5;
-	CU_ASSERT( lw_segment_intersects(p1, p2, q1, q2) == SEG_CROSS_RIGHT );
+	q1.x = -0.5;
+	q1.y = 0.5;
+	q2.x = 0.5;
+	q2.y = 0.5;
+	CU_ASSERT( lw_segment_intersects(&p1, &p2, &q1, &q2) == SEG_CROSS_RIGHT );
 
 	/* Q: Horizontal line crossing right to left */
-	q1->x = 0.5;
-	q1->y = 0.5;
-	q2->x = -0.5;
-	q2->y = 0.5;
-	CU_ASSERT( lw_segment_intersects(p1, p2, q1, q2) == SEG_CROSS_LEFT );
+	q1.x = 0.5;
+	q1.y = 0.5;
+	q2.x = -0.5;
+	q2.y = 0.5;
+	CU_ASSERT( lw_segment_intersects(&p1, &p2, &q1, &q2) == SEG_CROSS_LEFT );
 
 	/* Q: Horizontal line not crossing right to left */
-	q1->x = 0.5;
-	q1->y = 1.5;
-	q2->x = -0.5;
-	q2->y = 1.5;
-	CU_ASSERT( lw_segment_intersects(p1, p2, q1, q2) == SEG_NO_INTERSECTION );
+	q1.x = 0.5;
+	q1.y = 1.5;
+	q2.x = -0.5;
+	q2.y = 1.5;
+	CU_ASSERT( lw_segment_intersects(&p1, &p2, &q1, &q2) == SEG_NO_INTERSECTION );
 
-	/* Q: Horizontal line crossing at vertex right to left */
-	q1->x = 0.5;
-	q1->y = 1.0;
-	q2->x = -0.5;
-	q2->y = 1.0;
-	CU_ASSERT( lw_segment_intersects(p1, p2, q1, q2) == SEG_CROSS_LEFT );
+	/* Q: Horizontal line crossing at second vertex right to left */
+	q1.x = 0.5;
+	q1.y = 1.0;
+	q2.x = -0.5;
+	q2.y = 1.0;
+	CU_ASSERT( lw_segment_intersects(&p1, &p2, &q1, &q2) == SEG_NO_INTERSECTION );
 
-	/* Q: Diagonal line with large range crossing at vertex right to left */
-	q1->x = 0.5;
-	q1->y = 10.0;
-	q2->x = -0.5;
-	q2->y = -10.0;
-	CU_ASSERT( lw_segment_intersects(p1, p2, q1, q2) == SEG_CROSS_LEFT );
+	/* Q: Horizontal line crossing at first vertex right to left */
+	q1.x = 0.5;
+	q1.y = 0.0;
+	q2.x = -0.5;
+	q2.y = 0.0;
+	CU_ASSERT( lw_segment_intersects(&p1, &p2, &q1, &q2) == SEG_CROSS_LEFT );
 
-	/* Q: Diagonal line with large range crossing at vertex right to left */
-	q1->x = 0.5;
-	q1->y = 11.0;
-	q2->x = -0.5;
-	q2->y = -9.0;
-	CU_ASSERT( lw_segment_intersects(p1, p2, q1, q2) == SEG_CROSS_LEFT );
+	/* Q: Diagonal line with large range crossing at first vertex right to left */
+	q1.x = 0.5;
+	q1.y = 10.0;
+	q2.x = -0.5;
+	q2.y = -10.0;
+	CU_ASSERT( lw_segment_intersects(&p1, &p2, &q1, &q2) == SEG_CROSS_LEFT );
 
-	/* Q: Horizontal touching from left */
-	q1->x = -0.5;
-	q1->y = 0.5;
-	q2->x = 0.0;
-	q2->y = 0.5;
-	CU_ASSERT( lw_segment_intersects(p1, p2, q1, q2) == SEG_TOUCH_LEFT );
+	/* Q: Diagonal line with large range crossing at second vertex right to left */
+	q1.x = 0.5;
+	q1.y = 11.0;
+	q2.x = -0.5;
+	q2.y = -9.0;
+	CU_ASSERT( lw_segment_intersects(&p1, &p2, &q1, &q2) == SEG_NO_INTERSECTION );
 
-	/* Q: Horizontal touching from right */
-	q1->x = 0.5;
-	q1->y = 0.5;
-	q2->x = 0.0;
-	q2->y = 0.5;
-	CU_ASSERT( lw_segment_intersects(p1, p2, q1, q2) == SEG_TOUCH_RIGHT );
+	/* Q: Horizontal touching from left at second vertex*/
+	q1.x = -0.5;
+	q1.y = 0.5;
+	q2.x = 0.0;
+	q2.y = 0.5;
+	CU_ASSERT( lw_segment_intersects(&p1, &p2, &q1, &q2) == SEG_NO_INTERSECTION );
 
-	/* Q: Horizontal touching from left and far below*/
-	q1->x = -0.5;
-	q1->y = -10.5;
-	q2->x = 0.0;
-	q2->y = 0.5;
-	CU_ASSERT( lw_segment_intersects(p1, p2, q1, q2) == SEG_TOUCH_LEFT );
+	/* Q: Horizontal touching from right at first vertex */
+	q1.x = 0.0;
+	q1.y = 0.5;
+	q2.x = 0.5;
+	q2.y = 0.5;
+	CU_ASSERT( lw_segment_intersects(&p1, &p2, &q1, &q2) == SEG_CROSS_RIGHT );
 
-	/* Q: Horizontal touching from right and far above */
-	q1->x = 0.5;
-	q1->y = 10.5;
-	q2->x = 0.0;
-	q2->y = 0.5;
-	CU_ASSERT( lw_segment_intersects(p1, p2, q1, q2) == SEG_TOUCH_RIGHT );
+	/* Q: Horizontal touching from left and far below on second vertex */
+	q1.x = -0.5;
+	q1.y = -10.5;
+	q2.x = 0.0;
+	q2.y = 0.5;
+	CU_ASSERT( lw_segment_intersects(&p1, &p2, &q1, &q2) == SEG_NO_INTERSECTION );
+
+	/* Q: Horizontal touching from right and far above on second vertex */
+	q1.x = 0.5;
+	q1.y = 10.5;
+	q2.x = 0.0;
+	q2.y = 0.5;
+	CU_ASSERT( lw_segment_intersects(&p1, &p2, &q1, &q2) == SEG_NO_INTERSECTION );
 
 	/* Q: Co-linear from top */
-	q1->x = 0.0;
-	q1->y = 10.0;
-	q2->x = 0.0;
-	q2->y = 0.5;
-	CU_ASSERT( lw_segment_intersects(p1, p2, q1, q2) == SEG_COLINEAR );
+	q1.x = 0.0;
+	q1.y = 10.0;
+	q2.x = 0.0;
+	q2.y = 0.5;
+	CU_ASSERT( lw_segment_intersects(&p1, &p2, &q1, &q2) == SEG_COLINEAR );
 
 	/* Q: Co-linear from bottom */
-	q1->x = 0.0;
-	q1->y = -10.0;
-	q2->x = 0.0;
-	q2->y = 0.5;
-	CU_ASSERT( lw_segment_intersects(p1, p2, q1, q2) == SEG_COLINEAR );
+	q1.x = 0.0;
+	q1.y = -10.0;
+	q2.x = 0.0;
+	q2.y = 0.5;
+	CU_ASSERT( lw_segment_intersects(&p1, &p2, &q1, &q2) == SEG_COLINEAR );
 
 	/* Q: Co-linear contained */
-	q1->x = 0.0;
-	q1->y = 0.4;
-	q2->x = 0.0;
-	q2->y = 0.5;
-	CU_ASSERT( lw_segment_intersects(p1, p2, q1, q2) == SEG_COLINEAR );
+	q1.x = 0.0;
+	q1.y = 0.4;
+	q2.x = 0.0;
+	q2.y = 0.5;
+	CU_ASSERT( lw_segment_intersects(&p1, &p2, &q1, &q2) == SEG_COLINEAR );
 
 	/* Q: Horizontal touching at end point from left */
-	q1->x = -0.5;
-	q1->y = 1.0;
-	q2->x = 0.0;
-	q2->y = 1.0;
-	CU_ASSERT( lw_segment_intersects(p1, p2, q1, q2) == SEG_TOUCH_LEFT );
+	q1.x = -0.5;
+	q1.y = 1.0;
+	q2.x = 0.0;
+	q2.y = 1.0;
+	CU_ASSERT( lw_segment_intersects(&p1, &p2, &q1, &q2) == SEG_NO_INTERSECTION );
 
 	/* Q: Horizontal touching at end point from right */
-	q1->x = 0.5;
-	q1->y = 1.0;
-	q2->x = 0.0;
-	q2->y = 1.0;
-	CU_ASSERT( lw_segment_intersects(p1, p2, q1, q2) == SEG_TOUCH_RIGHT );
+	q1.x = 0.0;
+	q1.y = 1.0;
+	q2.x = 0.0;
+	q2.y = 0.5;
+	CU_ASSERT( lw_segment_intersects(&p1, &p2, &q1, &q2) == SEG_COLINEAR );
 
 	/* Q: Horizontal touching at start point from left */
-	q1->x = -0.5;
-	q1->y = 0.0;
-	q2->x = 0.0;
-	q2->y = 0.0;
-	CU_ASSERT( lw_segment_intersects(p1, p2, q1, q2) == SEG_TOUCH_LEFT );
+	q1.x = 0.0;
+	q1.y = 0.0;
+	q2.x = -0.5;
+	q2.y = 0.0;
+	CU_ASSERT( lw_segment_intersects(&p1, &p2, &q1, &q2) == SEG_CROSS_LEFT );
 
 	/* Q: Horizontal touching at start point from right */
-	q1->x = 0.5;
-	q1->y = 0.0;
-	q2->x = 0.0;
-	q2->y = 0.0;
-	CU_ASSERT( lw_segment_intersects(p1, p2, q1, q2) == SEG_TOUCH_RIGHT );
+	q1.x = 0.0;
+	q1.y = 0.0;
+	q2.x = 0.5;
+	q2.y = 0.0;
+	CU_ASSERT( lw_segment_intersects(&p1, &p2, &q1, &q2) == SEG_CROSS_RIGHT );
 
 }
 
 void test_lwline_crossing_short_lines(void)
 {
 
+	POINT4D p;
+
 	/*
-	** Simple test, two two-point lines 
+	** Simple test, two two-point lines
 	*/
 
 	/* Vertical line from 0,0 to 1,1 */
-	p->x = 0.0;
-	p->y = 0.0;
-	setPoint4d(pa21, 0, p);
-	p->y = 1.0;
-	setPoint4d(pa21, 1, p);
+	p.x = 0.0;
+	p.y = 0.0;
+	setPoint4d(pa21, 0, &p);
+	p.y = 1.0;
+	setPoint4d(pa21, 1, &p);
 
 	/* Horizontal, crossing mid-segment */
-	p->x = -0.5;
-	p->y = 0.5;
-	setPoint4d(pa22, 0, p);
-	p->x = 0.5;
-	setPoint4d(pa22, 1, p);
+	p.x = -0.5;
+	p.y = 0.5;
+	setPoint4d(pa22, 0, &p);
+	p.x = 0.5;
+	setPoint4d(pa22, 1, &p);
 
 	CU_ASSERT( lwline_crossing_direction(l21, l22) == LINE_CROSS_RIGHT );
 
-	/* Horizontal, crossing at top end vertex */
-	p->x = -0.5;
-	p->y = 1.0;
-	setPoint4d(pa22, 0, p);
-	p->x = 0.5;
-	setPoint4d(pa22, 1, p);
+	/* Horizontal, crossing at top end vertex (end crossings don't count) */
+	p.x = -0.5;
+	p.y = 1.0;
+	setPoint4d(pa22, 0, &p);
+	p.x = 0.5;
+	setPoint4d(pa22, 1, &p);
 
-	CU_ASSERT( lwline_crossing_direction(l21, l22) == LINE_CROSS_RIGHT );
+	CU_ASSERT( lwline_crossing_direction(l21, l22) == LINE_NO_CROSS );
 
 	/* Horizontal, crossing at bottom end vertex */
-	p->x = -0.5;
-	p->y = 0.0;
-	setPoint4d(pa22, 0, p);
-	p->x = 0.5;
-	setPoint4d(pa22, 1, p);
+	p.x = -0.5;
+	p.y = 0.0;
+	setPoint4d(pa22, 0, &p);
+	p.x = 0.5;
+	setPoint4d(pa22, 1, &p);
 
 	CU_ASSERT( lwline_crossing_direction(l21, l22) == LINE_CROSS_RIGHT );
 
 	/* Horizontal, no crossing */
-	p->x = -0.5;
-	p->y = 2.0;
-	setPoint4d(pa22, 0, p);
-	p->x = 0.5;
-	setPoint4d(pa22, 1, p);
+	p.x = -0.5;
+	p.y = 2.0;
+	setPoint4d(pa22, 0, &p);
+	p.x = 0.5;
+	setPoint4d(pa22, 1, &p);
 
 	CU_ASSERT( lwline_crossing_direction(l21, l22) == LINE_NO_CROSS );
 
 	/* Vertical, no crossing */
-	p->x = -0.5;
-	p->y = 0.0;
-	setPoint4d(pa22, 0, p);
-	p->y = 1.0;
-	setPoint4d(pa22, 1, p);
+	p.x = -0.5;
+	p.y = 0.0;
+	setPoint4d(pa22, 0, &p);
+	p.y = 1.0;
+	setPoint4d(pa22, 1, &p);
 
 	CU_ASSERT( lwline_crossing_direction(l21, l22) == LINE_NO_CROSS );
 
@@ -342,9 +335,8 @@ void test_lwline_crossing_long_lines(void)
 	LWLINE *l51;
 	LWLINE *l52;
 	/*
-	** More complex test, longer lines and multiple crossings 
+	** More complex test, longer lines and multiple crossings
 	*/
-
 	/* Vertical line with vertices at y integers */
 	l51 = (LWLINE*)lwgeom_from_ewkt("LINESTRING(0 0, 0 1, 0 2, 0 3, 0 4)", PARSER_CHECK_NONE);
 
@@ -373,11 +365,6 @@ void test_lwline_crossing_long_lines(void)
 	CU_ASSERT( lwline_crossing_direction(l51, l52) == LINE_MULTICROSS_END_SAME_FIRST_LEFT );
 	lwline_free(l52);
 
-	/* Two crossings, one at the last vertex on the next interior vertex */
-	l52 = (LWLINE*)lwgeom_from_ewkt("LINESTRING(1 4, 0 4, -1 3, 0 3, 1 3)", PARSER_CHECK_NONE);
-	CU_ASSERT( lwline_crossing_direction(l51, l52) == LINE_MULTICROSS_END_SAME_FIRST_LEFT );
-	lwline_free(l52);
-
 	/* Three crossings, two at midpoints, one at vertex */
 	l52 = (LWLINE*)lwgeom_from_ewkt("LINESTRING(0.5 1, -1 0.5, 1 2, -1 2, -1 3)", PARSER_CHECK_NONE);
 	CU_ASSERT( lwline_crossing_direction(l51, l52) == LINE_MULTICROSS_END_LEFT );
@@ -402,67 +389,83 @@ void test_lwline_crossing_long_lines(void)
 
 }
 
+
+void test_lwline_crossing_bugs(void)
+{
+	LWLINE *l1;
+	LWLINE *l2;
+
+	l1 = (LWLINE*)lwgeom_from_ewkt("LINESTRING(2.99 90.16,71 74,20 140,171 154)", PARSER_CHECK_NONE);
+	l2 = (LWLINE*)lwgeom_from_ewkt("LINESTRING(25 169,89 114,40 70,86 43)", PARSER_CHECK_NONE);
+
+	CU_ASSERT( lwline_crossing_direction(l1, l2) == LINE_MULTICROSS_END_RIGHT );
+	lwline_free(l1);
+	lwline_free(l2);
+
+}
+
 void test_lwpoint_set_ordinate(void)
 {
-	p->x = 0.0;
-	p->y = 0.0;
-	p->z = 0.0;
-	p->m = 0.0;
+	POINT4D p;
 
-	lwpoint_set_ordinate(p, 0, 1.5);
-	CU_ASSERT_EQUAL( p->x, 1.5 );
+	p.x = 0.0;
+	p.y = 0.0;
+	p.z = 0.0;
+	p.m = 0.0;
 
-	lwpoint_set_ordinate(p, 3, 2.5);
-	CU_ASSERT_EQUAL( p->m, 2.5 );
+	lwpoint_set_ordinate(&p, 0, 1.5);
+	CU_ASSERT_EQUAL( p.x, 1.5 );
 
-	lwpoint_set_ordinate(p, 2, 3.5);
-	CU_ASSERT_EQUAL( p->z, 3.5 );
+	lwpoint_set_ordinate(&p, 3, 2.5);
+	CU_ASSERT_EQUAL( p.m, 2.5 );
+
+	lwpoint_set_ordinate(&p, 2, 3.5);
+	CU_ASSERT_EQUAL( p.z, 3.5 );
 
 }
 
 void test_lwpoint_get_ordinate(void)
 {
+	POINT4D p;
 
-	p->x = 10.0;
-	p->y = 20.0;
-	p->z = 30.0;
-	p->m = 40.0;
+	p.x = 10.0;
+	p.y = 20.0;
+	p.z = 30.0;
+	p.m = 40.0;
 
-	CU_ASSERT_EQUAL( lwpoint_get_ordinate(p, 0), 10.0 );
-	CU_ASSERT_EQUAL( lwpoint_get_ordinate(p, 1), 20.0 );
-	CU_ASSERT_EQUAL( lwpoint_get_ordinate(p, 2), 30.0 );
-	CU_ASSERT_EQUAL( lwpoint_get_ordinate(p, 3), 40.0 );
+	CU_ASSERT_EQUAL( lwpoint_get_ordinate(&p, 0), 10.0 );
+	CU_ASSERT_EQUAL( lwpoint_get_ordinate(&p, 1), 20.0 );
+	CU_ASSERT_EQUAL( lwpoint_get_ordinate(&p, 2), 30.0 );
+	CU_ASSERT_EQUAL( lwpoint_get_ordinate(&p, 3), 40.0 );
 
 }
 
 void test_lwpoint_interpolate(void)
 {
-	POINT4D *r = NULL;
-	r = lwalloc(sizeof(POINT4D));
+	POINT4D p, q, r;
 	int rv = 0;
 
-	p->x = 10.0;
-	p->y = 20.0;
-	p->z = 30.0;
-	p->m = 40.0;
-	q->x = 20.0;
-	q->y = 30.0;
-	q->z = 40.0;
-	q->m = 50.0;
+	p.x = 10.0;
+	p.y = 20.0;
+	p.z = 30.0;
+	p.m = 40.0;
 
-	rv = lwpoint_interpolate(p, q, r, 4, 2, 35.0);
-	CU_ASSERT_EQUAL( r->x, 15.0);
+	q.x = 20.0;
+	q.y = 30.0;
+	q.z = 40.0;
+	q.m = 50.0;
 
-	rv = lwpoint_interpolate(p, q, r, 4, 3, 41.0);
-	CU_ASSERT_EQUAL( r->y, 21.0);
+	rv = lwpoint_interpolate(&p, &q, &r, 4, 2, 35.0);
+	CU_ASSERT_EQUAL( r.x, 15.0);
 
-	rv = lwpoint_interpolate(p, q, r, 4, 3, 50.0);
-	CU_ASSERT_EQUAL( r->y, 30.0);
+	rv = lwpoint_interpolate(&p, &q, &r, 4, 3, 41.0);
+	CU_ASSERT_EQUAL( r.y, 21.0);
 
-	rv = lwpoint_interpolate(p, q, r, 4, 3, 40.0);
-	CU_ASSERT_EQUAL( r->y, 20.0);
+	rv = lwpoint_interpolate(&p, &q, &r, 4, 3, 50.0);
+	CU_ASSERT_EQUAL( r.y, 30.0);
 
-	lwfree(r);
+	rv = lwpoint_interpolate(&p, &q, &r, 4, 3, 40.0);
+	CU_ASSERT_EQUAL( r.y, 20.0);
 
 }
 
@@ -584,7 +587,7 @@ void test_lwmline_clip(void)
 	LWLINE *line = NULL;
 
 	/*
-	** Set up the input line. Trivial one-member case. 
+	** Set up the input line. Trivial one-member case.
 	*/
 	mline = (LWMLINE*)lwgeom_from_ewkt("MULTILINESTRING((0 0,0 1,0 2,0 3,0 4))", PARSER_CHECK_NONE);
 
@@ -599,7 +602,7 @@ void test_lwmline_clip(void)
 	lwmline_free(mline);
 
 	/*
-	** Set up the input line. Two-member case. 
+	** Set up the input line. Two-member case.
 	*/
 	mline = (LWMLINE*)lwgeom_from_ewkt("MULTILINESTRING((1 0,1 1,1 2,1 3,1 4), (0 0,0 1,0 2,0 3,0 4))", PARSER_CHECK_NONE);
 
@@ -614,7 +617,7 @@ void test_lwmline_clip(void)
 	lwmline_free(mline);
 
 	/*
-	** Set up staggered input line to create multi-type output. 
+	** Set up staggered input line to create multi-type output.
 	*/
 	mline = (LWMLINE*)lwgeom_from_ewkt("MULTILINESTRING((1 0,1 -1,1 -2,1 -3,1 -4), (0 0,0 1,0 2,0 3,0 4))", PARSER_CHECK_NONE);
 
@@ -678,21 +681,22 @@ void test_lwline_clip_big(void)
 	LWLINE *line = lwline_construct(-1, NULL, pa);
 	LWCOLLECTION *c;
 	char *ewkt;
+	POINT4D p;
 
-	p->x = 0.0;
-	p->y = 0.0;
-	p->z = 0.0;
-	setPoint4d(pa, 0, p);
+	p.x = 0.0;
+	p.y = 0.0;
+	p.z = 0.0;
+	setPoint4d(pa, 0, &p);
 
-	p->x = 1.0;
-	p->y = 1.0;
-	p->z = 1.0;
-	setPoint4d(pa, 1, p);
+	p.x = 1.0;
+	p.y = 1.0;
+	p.z = 1.0;
+	setPoint4d(pa, 1, &p);
 
-	p->x = 2.0;
-	p->y = 2.0;
-	p->z = 2.0;
-	setPoint4d(pa, 2, p);
+	p.x = 2.0;
+	p.y = 2.0;
+	p.z = 2.0;
+	setPoint4d(pa, 2, &p);
 
 	c = lwline_clip_to_ordinate_range(line, 2, 0.5, 1.5);
 	ewkt = lwgeom_to_ewkt((LWGEOM*)c, PARSER_CHECK_NONE);
