@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: cu_algorithm.h 4168 2009-06-11 16:44:03Z pramsey $
+ * $Id: cu_algorithm.h 4786 2009-11-11 19:02:19Z pramsey $
  *
  * PostGIS - Spatial Types for PostgreSQL
  * http://postgis.refractions.net
@@ -16,16 +16,12 @@
 #include "CUnit/Basic.h"
 
 #include "lwalgorithm.h"
+#include "cu_tester.h"
 
 /***********************************************************************
 ** for Computational Geometry Suite
 */
 
-/* Set-up / clean-up functions */
-CU_pSuite register_cg_suite(void);
-int init_cg_suite(void);
-int clean_cg_suite(void);
-
 /* Test functions */
 void test_lw_segment_side(void);
 void test_lw_segment_intersects(void);
@@ -40,3 +36,4 @@ void test_lwmline_clip(void);
 void test_geohash_precision(void);
 void test_geohash_point(void);
 void test_geohash(void);
+void test_lwline_crossing_bugs(void);
