@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: cu_tester.c 4168 2009-06-11 16:44:03Z pramsey $
+ * $Id: cu_tester.c 4508 2009-09-17 05:33:35Z pramsey $
  *
  * PostGIS - Spatial Types for PostgreSQL
  * http://postgis.refractions.net
@@ -13,9 +13,8 @@
 #include <stdio.h>
 #include <string.h>
 #include "CUnit/Basic.h"
-
-#include "cu_algorithm.h"
-#include "cu_measures.h"
+#include "liblwgeom.h"
+#include "cu_tester.h"
 
 
 /*
@@ -54,6 +53,20 @@ int main()
 		return CU_get_error();
 	}
 
+	/* Add the libgeom suite to the registry */
+	if (NULL == register_libgeom_suite())
+	{
+		CU_cleanup_registry();
+		return CU_get_error();
+	}
+
+	/* Add the geodetic suite to the registry */
+	if (NULL == register_geodetic_suite())
+	{
+		CU_cleanup_registry();
+		return CU_get_error();
+	}
+
 	/* Run all tests using the CUnit Basic interface */
 	CU_basic_set_mode(CU_BRM_VERBOSE);
 	CU_basic_run_tests();
