@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: lwcircstring.c 4894 2009-11-25 19:15:57Z pramsey $
+ * $Id: lwcircstring.c 7464 2011-06-24 03:51:11Z pramsey $
  *
  * PostGIS - Spatial Types for PostgreSQL
  * http://postgis.refractions.net
@@ -17,6 +17,7 @@
 #include <string.h>
 #include <math.h>
 #include "liblwgeom.h"
+#include "lwalgorithm.h"
 
 BOX3D *lwcircle_compute_box3d(POINT4D *p1, POINT4D *p2, POINT4D *p3);
 void printLWCIRCSTRING(LWCIRCSTRING *curve);
@@ -258,6 +259,96 @@ lwcircstring_serialize_size(LWCIRCSTRING *curve)
 }
 
 
+static inline int signum(double n)
+{
+	if( n < 0 ) return -1;
+	if( n > 0 ) return 1;
+	return 0;
+}
+
+/*
+static int lwcircle_calculate_gbox_cartesian(const POINT4D *p1, const POINT4D *p2, const POINT4D *p3, GBOX *gbox)
+*/
+BOX3D *
+lwcircle_compute_box3d(POINT4D *p1, POINT4D *p2, POINT4D *p3)
+{
+	POINT2D xmin, ymin, xmax, ymax;
+	POINT4D *center = NULL;
+	int p2_side = 0;
+	double radius = 0.0;
+
+	LWDEBUG(2, "lwcircle_compute_box3d called.");
+
+	radius = lwcircle_center(p1, p2, p3, &center);
+	BOX3D *box = lwalloc(sizeof(BOX3D));
+	
+	/* Negative radius signals straight line, p1/p2/p3 are colinear */
+	if (radius < 0.0)
+	{
+		if ( center ) lwfree(center);
+        box->xmin = FP_MIN(p1->x, p3->x);
+        box->ymin = FP_MIN(p1->y, p3->y);
+        box->zmin = FP_MIN(p1->z, p3->z);
+        box->xmax = FP_MAX(p1->x, p3->x);
+        box->ymax = FP_MAX(p1->y, p3->y);
+        box->zmax = FP_MAX(p1->z, p3->z);
+	    return box;
+	}
+	
+	/* Matched start/end points imply circle */
+	if ( p1->x == p3->x && p1->y == p3->y )
+	{
+		box->xmin = center->x - radius;
+		box->ymin = center->y - radius;
+		box->zmin = FP_MIN(p1->z,p2->z);
+		box->xmax = center->x + radius;
+		box->ymax = center->y + radius;
+		box->zmax = FP_MAX(p1->z,p2->z);
+		lwfree(center);
+		return box;
+	}
+
+	/* First approximation, bounds of start/end points */
+    box->xmin = FP_MIN(p1->x, p3->x);
+    box->ymin = FP_MIN(p1->y, p3->y);
+    box->zmin = FP_MIN(p1->z, p3->z);
+    box->xmax = FP_MAX(p1->x, p3->x);
+    box->ymax = FP_MAX(p1->y, p3->y);
+    box->zmax = FP_MAX(p1->z, p3->z);
+
+	/* Create points for the possible extrema */
+	xmin.x = center->x - radius;
+	xmin.y = center->y;
+	ymin.x = center->x;
+	ymin.y = center->y - radius;
+	xmax.x = center->x + radius;
+	xmax.y = center->y;
+	ymax.x = center->x;
+	ymax.y = center->y + radius;
+	
+	/* Divide the circle into two parts, one on each side of a line
+	   joining p1 and p3. The circle extrema on the same side of that line
+	   as p2 is on, are also the extrema of the bbox. */
+	
+	p2_side = signum(lw_segment_side((POINT2D*)p1, (POINT2D*)p3, (POINT2D*)p2));
+
+	if ( p2_side == signum(lw_segment_side((POINT2D*)p1, (POINT2D*)p3, &xmin)) )
+		box->xmin = xmin.x;
+
+	if ( p2_side == signum(lw_segment_side((POINT2D*)p1, (POINT2D*)p3, &ymin)) )
+		box->ymin = ymin.y;
+
+	if ( p2_side == signum(lw_segment_side((POINT2D*)p1, (POINT2D*)p3, &xmax)) )
+		box->xmax = xmax.x;
+
+	if ( p2_side == signum(lw_segment_side((POINT2D*)p1, (POINT2D*)p3, &ymax)) )
+		box->ymax = ymax.y;
+
+	lwfree(center);
+	return box;
+}
+
+#if 0
 BOX3D *
 lwcircle_compute_box3d(POINT4D *p1, POINT4D *p2, POINT4D *p3)
 {
@@ -274,9 +365,37 @@ lwcircle_compute_box3d(POINT4D *p1, POINT4D *p2, POINT4D *p3)
 
 	LWDEBUG(2, "lwcircle_compute_box3d called.");
 
-	center = lwalloc(sizeof(POINT4D));
 	radius = lwcircle_center(p1, p2, p3, &center);
-	if (radius < 0.0) return NULL;
+	if (radius < 0.0) 
+	{
+		LWDEBUG(3, "lwcircle_compute_box3d: zero radius");
+
+		/* 
+		 * We've got a straight line here.  Look to the end points for extents. 
+		 * It's worth noting that when lwcircle_center returns < 0, center hasn't been allocated.
+		 */
+
+		x1 = (FP_LT(p1->x, p3->x)) ? p1->x : p3->x;
+		x2 = (FP_GT(p1->x, p3->x)) ? p1->x : p3->x;
+		y1 = (FP_LT(p1->y, p3->y)) ? p1->y : p3->y;
+		y2 = (FP_GT(p1->y, p3->y)) ? p1->y : p3->y;
+		z1 = (FP_LT(p1->z, p2->z)) ? p1->z : p2->z;
+		z1 = (FP_LT(z1, p3->z)) ? z1 : p3->z;
+		z2 = (FP_GT(p1->z, p2->z)) ? p1->z : p2->z;
+		z2 = (FP_GT(z2, p3->z)) ? z2 : p3->z;
+
+		box = lwalloc(sizeof(BOX3D));
+		box->xmin = x1;
+		box->xmax = x2;
+		box->ymin = y1;
+		box->ymax = y2;
+		box->zmin = z1;
+		box->zmax = z2;
+
+		LWDEBUGF(3, "lwcircle_compute_box3d: extents %.16f %.16f %.16f, %.16f %.16f %.16f", x1, y1, z1, x2, y2, z2);
+
+		return box;
+	}
 
 	/*
 	top = center->y + radius;
@@ -298,6 +417,9 @@ lwcircle_compute_box3d(POINT4D *p1, POINT4D *p2, POINT4D *p3)
 	r2 = a2 - a1;
 	r3 = a3 - a1;
 
+	LWDEBUGF(4, "a1 %.16f, a2 %.16f, a3 %.16f", a1, a2, a3);
+	LWDEBUGF(4, "r2 %.16f, r3 %.16f", r2, r3);
+
 	/*
 	 * There are six cases here I'm interested in
 	 * Clockwise:
@@ -448,6 +570,7 @@ lwcircle_compute_box3d(POINT4D *p1, POINT4D *p2, POINT4D *p3)
 
 	return box;
 }
+#endif
 
 /*
  * Find bounding box (standard one)
