@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: lwmpoint.c 4168 2009-06-11 16:44:03Z pramsey $
+ * $Id: lwmpoint.c 5181 2010-02-01 17:35:55Z pramsey $
  *
  * PostGIS - Spatial Types for PostgreSQL
  * http://postgis.refractions.net
@@ -43,14 +43,25 @@ lwmpoint_deserialize(uchar *srl)
 	result->type = insp->type;
 	result->SRID = insp->SRID;
 	result->ngeoms = insp->ngeometries;
-	result->geoms = lwalloc(sizeof(LWPOINT *)*result->ngeoms);
+
+	if ( insp->ngeometries )
+	{
+		result->geoms = lwalloc(sizeof(LWPOINT *)*insp->ngeometries);
+	}
+	else
+	{
+		result->geoms = NULL;
+	}
 
 	if (lwgeom_hasBBOX(srl[0]))
 	{
 		result->bbox = lwalloc(sizeof(BOX2DFLOAT4));
 		memcpy(result->bbox, srl+1, sizeof(BOX2DFLOAT4));
 	}
-	else result->bbox = NULL;
+	else
+	{
+		result->bbox = NULL;
+	}
 
 	for (i=0; i<insp->ngeometries; i++)
 	{
@@ -135,5 +146,5 @@ void lwmpoint_free(LWMPOINT *mpt)
 	}
 	lwfree(mpt);
 
-};
+}
 
