@@ -21,24 +21,24 @@ lwreporter lwnotice_var = init_noticereporter;
 lwreporter lwerror_var = init_errorreporter;
 
 static char *lwgeomTypeName[] =
-    {
-        "Unknown",
-        "Point",
-        "Line",
-        "Polygon",
-        "MultiPoint",
-        "MultiLine",
-        "MultiPolygon",
-        "GeometryCollection",
-        "CircularString",
-        "CompoundString",
-        "Invalid Type",  /* POINTTYPEI */
-        "Invalid Type",  /* LINETYPEI */
-        "Invalid Type",  /* POLYTYPEI */
-        "CurvePolygon",
-        "MultiCurve",
-        "MultiSurface"
-    };
+{
+	"Unknown",
+	"Point",
+	"LineString",
+	"Polygon",
+	"MultiPoint",
+	"MultiLineString",
+	"MultiPolygon",
+	"GeometryCollection",
+	"CircularString",
+	"CompoundString",
+	"Invalid Type",  /* POINTTYPEI */
+	"Invalid Type",  /* LINETYPEI */
+	"Invalid Type",  /* POLYTYPEI */
+	"CurvePolygon",
+	"MultiCurve",
+	"MultiSurface"
+};
 
 
 /*
@@ -207,8 +207,7 @@ void lwgeom_install_default_allocators(void)
 }
 
 
-const char *
-lwgeom_typename(int type)
+const char* lwgeom_typename(int type)
 {
 	/* something went wrong somewhere */
 	if ( type < 0 || type > 15 )
