@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: shp2pgsql-cli.c 5194 2010-02-03 21:42:20Z pramsey $
+ * $Id: shp2pgsql-cli.c 6373 2010-12-14 04:57:59Z pramsey $
  *
  * PostGIS - Spatial Types for PostgreSQL
  * http://postgis.refractions.net
@@ -31,17 +31,18 @@ usage()
 	printf("      -c  Creates a new table and populates it, this is the\n");
 	printf("          default if you do not specify any options.\n");
 	printf("      -p  Prepare mode, only creates the table.\n");
-	printf("  -g <geocolumn> Specify the name of the geometry/geography column\n");
-	printf("     (mostly useful in append mode).\n");
-	printf("  -D  Use postgresql dump format (defaults to SQL insert statments.\n");
+	printf("  -g <geocolumn> Specify the name of the geometry/geography column.\n");
+	printf("      (mostly useful in append mode).\n");
+	printf("  -D  Use postgresql dump format (defaults to SQL insert statments).\n");
 	printf("  -G  Use geography type (requires lon/lat data).\n");
 	printf("  -k  Keep postgresql identifiers case.\n");
 	printf("  -i  Use int4 type for all integer dbf fields.\n");
 	printf("  -I  Create a spatial index on the geocolumn.\n");
 	printf("  -S  Generate simple geometries instead of MULTI geometries.\n");
+	printf("  -w  Output WKT format (drops M and introduces coordinate drifts).\n");
 	printf("  -W <encoding> Specify the character encoding of Shape's\n");
-	printf("     attribute column. (default : \"WINDOWS-1252\")\n");
-	printf("  -N <policy> NULL geometries handling policy (insert*,skip,abort)\n");
+	printf("      attribute column. (default : \"WINDOWS-1252\").\n");
+	printf("  -N <policy> NULL geometries handling policy (insert*,skip,abort).\n");
 	printf("  -n  Only import DBF file.\n");
 	printf("  -?  Display this help screen.\n");
 }
@@ -92,9 +93,9 @@ main (int argc, char **argv)
 			break;
 
 		case 's':
-			if (optarg)
+			if (pgis_optarg)
 			{
-				sscanf(optarg, "%d", &(config->sr_id));
+				sscanf(pgis_optarg, "%d", &(config->sr_id));
 			}
 			else
 			{
@@ -105,7 +106,7 @@ main (int argc, char **argv)
 			break;
 
 		case 'g':
-			config->geom = optarg;
+			config->geom = pgis_optarg;
 			break;
 
 		case 'k':
@@ -129,11 +130,11 @@ main (int argc, char **argv)
 			break;
 
 		case 'W':
-			config->encoding = optarg;
+			config->encoding = pgis_optarg;
 			break;
 
 		case 'N':
-			switch (optarg[0])
+			switch (pgis_optarg[0])
 			{
 			case 'a':
 				config->null_policy = POLICY_NULL_ABORT;
@@ -161,10 +162,10 @@ main (int argc, char **argv)
 	}
 
 	/* Determine the shapefile name from the next argument, if no shape file, exit. */
-	if (optind < argc)
+	if (pgis_optind < argc)
 	{
-		config->shp_file = argv[optind];
-		optind++;
+		config->shp_file = argv[pgis_optind];
+		pgis_optind++;
 	}
 	else
 	{
@@ -173,25 +174,25 @@ main (int argc, char **argv)
 	}
 
 	/* Determine the table and schema names from the next argument */
-	if (optind < argc)
+	if (pgis_optind < argc)
 	{
 		char *ptr;
 
-		ptr = strchr(argv[optind], '.');
+		ptr = strchr(argv[pgis_optind], '.');
 
 		/* Schema qualified table name */
 		if (ptr)
 		{
-			config->schema = malloc(strlen(argv[optind]) + 1);
-			snprintf(config->schema, ptr - argv[optind] + 1, "%s", argv[optind]);
+			config->schema = malloc(strlen(argv[pgis_optind]) + 1);
+			snprintf(config->schema, ptr - argv[pgis_optind] + 1, "%s", argv[pgis_optind]);
 
-			config->table = malloc(strlen(argv[optind]));
-			snprintf(config->table, strlen(argv[optind]) - strlen(config->schema), "%s", ptr + 1);
+			config->table = malloc(strlen(argv[pgis_optind]));
+			snprintf(config->table, strlen(argv[pgis_optind]) - strlen(config->schema), "%s", ptr + 1);
 		}
 		else
 		{
-			config->table = malloc(strlen(argv[optind]) + 1);
-			strcpy(config->table, argv[optind]);
+			config->table = malloc(strlen(argv[pgis_optind]) + 1);
+			strcpy(config->table, argv[pgis_optind]);
 		}
 	}
 
