@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: shp2pgsql-core.c 5195 2010-02-03 22:42:13Z pramsey $
+ * $Id: shp2pgsql-core.c 6361 2010-12-13 20:42:47Z pramsey $
  *
  * PostGIS - Spatial Types for PostgreSQL
  * http://postgis.refractions.net
@@ -37,11 +37,22 @@ void lwgeom_init_allocators()
 
 
 /*
- * Internal functions
- */
+* Internal return values
+*/
 
-char *utf8(const char *fromcode, char *inputbuf);
-void vasbappend(stringbuffer_t *sb, char *fmt, ... );
+#define UTF8_GOOD_RESULT 0
+#define UTF8_BAD_RESULT 1
+#define UTF8_NO_RESULT 2
+
+/*
+* Only turn this on if you want to skip bad UTF8 charaters. Not a good
+* idea, generally, as "bad characters" usually indicate that the source
+* encoding is *not* UTF8
+*/
+#define UTF8_DROP_BAD_CHARACTERS 0
+
+
+int utf8(const char *fromcode, char *inputbuf, char **outputbuf);
 char *escape_copy_string(char *str);
 char *escape_insert_string(char *str);
 
@@ -76,12 +87,10 @@ vasbappend(stringbuffer_t *sb, char *fmt, ... )
 }
 
 /* Return allocated string containing UTF8 string converted from encoding fromcode */
-char *
-utf8(const char *fromcode, char *inputbuf)
+int utf8(const char *fromcode, char *inputbuf, char **outputbuf)
 {
 	iconv_t cd;
 	char *outputptr;
-	char *outputbuf;
 	size_t outbytesleft;
 	size_t inbytesleft;
 
@@ -89,23 +98,48 @@ utf8(const char *fromcode, char *inputbuf)
 
 	cd = iconv_open("UTF-8", fromcode);
 	if ( cd == ((iconv_t)(-1)) )
-		return NULL;
+		return UTF8_NO_RESULT;
 
 	outbytesleft = inbytesleft * 3 + 1; /* UTF8 string can be 3 times larger */
-	/* then local string */
-	outputbuf = (char *)malloc(outbytesleft);
-	if (!outputbuf)
-		return NULL;
 
-	memset(outputbuf, 0, outbytesleft);
-	outputptr = outputbuf;
+	*outputbuf = (char *)malloc(outbytesleft);
+	if (!(*outputbuf))
+		return UTF8_NO_RESULT;
 
-	if (-1 == iconv(cd, &inputbuf, &inbytesleft, &outputptr, &outbytesleft))
-		return NULL;
+    /* Clean out the buffer */
+	memset(*outputbuf, 0, outbytesleft);
+	outputptr = *outputbuf;
 
-	iconv_close (cd);
-
-	return outputbuf;
+    /* Does this string convert cleanly? */
+	if ( iconv(cd, &inputbuf, &inbytesleft, &outputptr, &outbytesleft) == -1 )
+	{
+#ifdef HAVE_ICONVCTL
+        int on = 1;
+	    /* No. Try to convert it while transliterating. */
+        iconvctl(cd, ICONV_SET_TRANSLITERATE, &on);
+    	if ( iconv(cd, &inputbuf, &inbytesleft, &outputptr, &outbytesleft) == -1 )
+    	{
+	        /* No. Try to convert it while discarding errors. */
+            iconvctl(cd, ICONV_SET_DISCARD_ILSEQ, &on);
+        	if ( iconv(cd, &inputbuf, &inbytesleft, &outputptr, &outbytesleft) == -1 )
+        	{
+                /* Still no. Throw away the buffer and return. */
+                free(*outputbuf);
+                iconv_close(cd);
+                return UTF8_NO_RESULT;
+            }
+        }
+        iconv_close(cd);
+        return UTF8_BAD_RESULT;
+#else
+        free(*outputbuf);
+        iconv_close(cd);
+        return UTF8_NO_RESULT;        
+#endif
+    }
+    /* Return a good result, converted string is in buffer. */
+	iconv_close(cd);
+    return UTF8_GOOD_RESULT;
 }
 
 /**
@@ -216,6 +250,62 @@ escape_insert_string(char *str)
 }
 
 
+/**
+ * Escape strings that are to be used as part of a PostgreSQL connection string. If no 
+ * characters require escaping, simply return the input pointer. Otherwise return a 
+ * new allocated string.
+ */
+char *
+escape_connection_string(char *str)
+{
+	/*
+	 * Escape apostrophes and backslashes:
+	 *   ' -> \'
+	 *   \ -> \\
+	 *
+	 * 1. find # of characters
+	 * 2. make new string
+	 */
+
+	char *result;
+	char *ptr, *optr;
+	int toescape = 0;
+	size_t size;
+
+	ptr = str;
+
+	/* Count how many characters we need to escape so we know the size of the string we need to return */
+	while (*ptr)
+	{
+		if (*ptr == '\'' || *ptr == '\\')
+			toescape++;
+
+		ptr++;
+	}
+
+	/* If we don't have to escape anything, simply return the input pointer */
+	if (toescape == 0)
+		return str;
+
+	size = ptr - str + toescape + 1;
+	result = calloc(1, size);
+	optr = result;
+	ptr = str;
+
+	while (*ptr)
+	{
+		if (*ptr == '\'' || *ptr == '\\')
+			*optr++ = '\\';
+
+		*optr++ = *ptr++;
+	}
+
+	*optr = '\0';
+
+	return result;
+}
+
+
 /**
  * @brief Generate an allocated geometry string for shapefile object obj using the state parameters
  */
@@ -406,6 +496,14 @@ GenerateLineStringGeometry(SHPLOADERSTATE *state, SHPObject *obj, char **geometr
 	if (state->config->simple_geometries == 0)
 	{
 		lwcollection = lwcollection_construct(MULTILINETYPE, state->config->sr_id, NULL, obj->nParts, lwmultilinestrings);
+
+		/* When outputting wkt rather than wkb, we need to remove the SRID from the inner geometries */
+		if (state->config->hwgeom)
+		{
+			for (u = 0; u < obj->nParts; u++)
+				lwmultilinestrings[u]->SRID = -1;
+		}
+
 		serialized_lwgeom = lwgeom_serialize(lwcollection_as_lwgeom(lwcollection));
 	}
 	else
@@ -762,6 +860,14 @@ GeneratePolygonGeometry(SHPLOADERSTATE *state, SHPObject *obj, char **geometry)
 	if (state->config->simple_geometries == 0)
 	{
 		lwcollection = lwcollection_construct(MULTIPOLYGONTYPE, state->config->sr_id, NULL, polygon_total, lwpolygons);
+
+		/* When outputting wkt rather than wkb, we need to remove the SRID from the inner geometries */
+		if (state->config->hwgeom)
+		{
+			for (u = 0; u < pi; u++)
+				lwpolygons[u]->SRID = -1;
+		}
+
 		serialized_lwgeom = lwgeom_serialize(lwcollection_as_lwgeom(lwcollection));
 	}
 	else
@@ -875,6 +981,10 @@ ShpLoaderCreate(SHPLOADERCONFIG *config)
 	state->hSHPHandle = NULL;
 	state->hDBFHandle = NULL;
 	state->wkbtype = 0;
+    state->types = NULL;
+    state->widths = NULL;
+    state->precisions = NULL;
+    state->col_names = NULL;
 
 	return state;
 }
@@ -1147,11 +1257,22 @@ ShpLoaderOpenShape(SHPLOADERSTATE *state)
 
 		if (state->config->encoding)
 		{
-			/* If we are converting from another encoding to UTF8, convert the field name to UTF8 */
-			utf8str = utf8(state->config->encoding, name);
-			if (!utf8str)
+            static char *encoding_msg = "Try \"LATIN1\" (Western European), or one of the values described at http://www.postgresql.org/docs/current/static/multibyte.html.";
+
+            int rv = utf8(state->config->encoding, name, &utf8str);
+						
+			if (rv != UTF8_GOOD_RESULT)
 			{
-				snprintf(state->message, SHPLOADERMSGLEN, "Unable to convert field name \"%s\" to UTF-8: iconv reports \"%s\"", name, strerror(errno));
+                if( rv == UTF8_BAD_RESULT )
+				    snprintf(state->message, SHPLOADERMSGLEN, "Unable to convert field name \"%s\" to UTF-8 (iconv reports \"%s\"). Current encoding is \"%s\". %s", utf8str, strerror(errno), state->config->encoding, encoding_msg);
+			    else if( rv == UTF8_NO_RESULT )
+				    snprintf(state->message, SHPLOADERMSGLEN, "Unable to convert field name to UTF-8 (iconv reports \"%s\"). Current encoding is \"%s\". %s", strerror(errno), state->config->encoding, encoding_msg);
+				else 
+				    snprintf(state->message, SHPLOADERMSGLEN, "Unexpected return value from utf8()");
+
+                if( rv == UTF8_BAD_RESULT )
+			        free(utf8str);
+
 				return SHPLOADERERR;
 			}
 
@@ -1463,9 +1584,10 @@ ShpLoaderGenerateSQLRowStatement(SHPLOADERSTATE *state, int item, char **strreco
 	stringbuffer_t *sbwarn;
 	char val[MAXVALUELEN];
 	char *escval;
-	char *geometry, *ret;
+	char *geometry=NULL, *ret;
 	char *utf8str;
 	int res, i;
+	int rv;
 
 	/* Clear the stringbuffers */
 	sbwarn = stringbuffer_create();
@@ -1534,7 +1656,8 @@ ShpLoaderGenerateSQLRowStatement(SHPLOADERSTATE *state, int item, char **strreco
 			{
 			case FTInteger:
 			case FTDouble:
-				if (-1 == snprintf(val, MAXVALUELEN, "%s", DBFReadStringAttribute(state->hDBFHandle, item, i)))
+				rv = snprintf(val, MAXVALUELEN, "%s", DBFReadStringAttribute(state->hDBFHandle, item, i));
+				if (rv >= MAXVALUELEN || rv == -1)
 				{
 					vasbappend(sbwarn, "Warning: field %d name truncated\n", i);
 					val[MAXVALUELEN - 1] = '\0';
@@ -1555,7 +1678,8 @@ ShpLoaderGenerateSQLRowStatement(SHPLOADERSTATE *state, int item, char **strreco
 			case FTString:
 			case FTLogical:
 			case FTDate:
-				if (-1 == snprintf(val, MAXVALUELEN, "%s", DBFReadStringAttribute(state->hDBFHandle, item, i)))
+				rv = snprintf(val, MAXVALUELEN, "%s", DBFReadStringAttribute(state->hDBFHandle, item, i));
+				if (rv >= MAXVALUELEN || rv == -1)
 				{
 					vasbappend(sbwarn, "Warning: field %d name truncated\n", i);
 					val[MAXVALUELEN - 1] = '\0';
@@ -1574,16 +1698,37 @@ ShpLoaderGenerateSQLRowStatement(SHPLOADERSTATE *state, int item, char **strreco
 
 			if (state->config->encoding)
 			{
+                static char *encoding_msg = "Try \"LATIN1\" (Western European), or one of the values described at http://www.postgresql.org/docs/current/static/multibyte.html.";
 				/* If we are converting from another encoding to UTF8, convert the field value to UTF8 */
-				utf8str = utf8(state->config->encoding, val);
-				if (!utf8str)
+				int rv = utf8(state->config->encoding, val, &utf8str);
+                if ( !UTF8_DROP_BAD_CHARACTERS && rv != UTF8_GOOD_RESULT )
+                {
+                    if( rv == UTF8_BAD_RESULT )
+					    snprintf(state->message, SHPLOADERMSGLEN, "Unable to convert data value \"%s\" to UTF-8 (iconv reports \"%s\"). Current encoding is \"%s\". %s", utf8str, strerror(errno), state->config->encoding, encoding_msg);
+				    else if( rv == UTF8_NO_RESULT )
+					    snprintf(state->message, SHPLOADERMSGLEN, "Unable to convert data value to UTF-8 (iconv reports \"%s\"). Current encoding is \"%s\". %s", strerror(errno), state->config->encoding, encoding_msg);
+					else 
+					    snprintf(state->message, SHPLOADERMSGLEN, "Unexpected return value from utf8()");
+
+                    if( rv == UTF8_BAD_RESULT )
+				        free(utf8str);
+        		    
+                	return SHPLOADERERR;
+                }
+				/* Optionally (compile-time) suppress bad UTF8 values */
+				if ( UTF8_DROP_BAD_CHARACTERS && rv != UTF8_GOOD_RESULT )
 				{
-					snprintf(state->message, SHPLOADERMSGLEN, "Unable to convert field value \"%s\" to UTF-8: iconv reports \"%s\"", val, strerror(errno));
-					return SHPLOADERERR;
+					val[0] = '.';
+					val[1] = '\0';
 				}
 
-				strncpy(val, utf8str, MAXVALUELEN);
-				free(utf8str);
+				
+				/* The utf8str buffer is only alloc'ed if the UTF8 conversion works */
+				if ( rv == UTF8_GOOD_RESULT )
+				{
+					strncpy(val, utf8str, MAXVALUELEN);
+					free(utf8str);
+				}
 			}
 
 			/* Escape attribute correctly according to dump format */
@@ -1689,50 +1834,51 @@ ShpLoaderGenerateSQLRowStatement(SHPLOADERSTATE *state, int item, char **strreco
 
 				return SHPLOADERERR;
 			}
-		}
 
 
-		/* Now generate the geometry string according to the current configuration */
-		if (state->config->hwgeom)
-		{
-			/* Old-style hwgeom (WKT) */
-			if (!state->config->dump_format)
-				vasbappend(sb, "GeomFromText('");
+			/* Now generate the geometry string according to the current configuration */
+			if (state->config->hwgeom)
+			{
+				/* Old-style hwgeom (WKT) */
+				if (!state->config->dump_format)
+					vasbappend(sb, "GeomFromText('");
+				else
+				{
+					/* Output SRID if relevant */
+					if (state->config->sr_id != 0)
+						vasbappend(sb, "SRID=%d;", state->config->sr_id);
+				}
+
+				vasbappend(sb, "%s", geometry);
+
+				if (!state->config->dump_format)
+				{
+					vasbappend(sb, "'");
+
+					/* Output SRID if relevant */
+					if (state->config->sr_id != 0)
+						vasbappend(sb, ", %d)", state->config->sr_id);
+					else
+						vasbappend(sb, ")");
+				}
+			}
 			else
 			{
-				/* Output SRID if relevant */
-				if (state->config->sr_id != 0)
-					vasbappend(sb, "SRID=%d;", state->config->sr_id);
+				/* New style lwgeom (HEXEWKB) */
+				if (!state->config->dump_format)
+					vasbappend(sb, "'");
+
+				vasbappend(sb, "%s", geometry);
+
+				if (!state->config->dump_format)
+					vasbappend(sb, "'");
 			}
 
-			vasbappend(sb, "%s", geometry);
-
-			if (!state->config->dump_format)
-			{
-				vasbappend(sb, "'");
-
-				/* Output SRID if relevant */
-				if (state->config->sr_id != 0)
-					vasbappend(sb, ", %d)", state->config->sr_id);
-				else
-					vasbappend(sb, ")");
-			}
-		}
-		else
-		{
-			/* New style lwgeom (HEXEWKB) */
-			if (!state->config->dump_format)
-				vasbappend(sb, "'");
-
-			vasbappend(sb, "%s", geometry);
-
-			if (!state->config->dump_format)
-				vasbappend(sb, "'");
+			free(geometry);
 		}
 
 		/* Tidy up everything */
 		SHPDestroyObject(obj);
-		free(geometry);
 	}
 
 	/* Close the line correctly for dump/insert format */
