@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: shp2pgsql-core.h 5098 2010-01-04 05:47:04Z pramsey $
+ * $Id: shp2pgsql-core.h 5983 2010-09-19 11:27:05Z mcayland $
  *
  * PostGIS - Spatial Types for PostgreSQL
  * http://postgis.refractions.net
@@ -29,7 +29,7 @@
 
 #include "stringbuffer.h"
 
-#define RCSID "$Id: shp2pgsql-core.h 5098 2010-01-04 05:47:04Z pramsey $"
+#define RCSID "$Id: shp2pgsql-core.h 5983 2010-09-19 11:27:05Z mcayland $"
 
 
 /*
@@ -70,7 +70,7 @@
 /*
  * Default character encoding
  */
-#define ENCODING_DEFAULT "WINDOWS-1252"
+#define ENCODING_DEFAULT "UTF-8"
 
 /*
  * Structure to hold the loader configuration options 
@@ -208,6 +208,8 @@ typedef struct shp_connection_state
 
 /* Externally accessible functions */
 void strtolower(char *s);
+void vasbappend(stringbuffer_t *sb, char *fmt, ... );
+char *escape_connection_string(char *str);
 void set_config_defaults(SHPLOADERCONFIG *config);
 
 SHPLOADERSTATE *ShpLoaderCreate(SHPLOADERCONFIG *config);
