@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: shp2pgsql-gui.c 5181 2010-02-01 17:35:55Z pramsey $
+ * $Id: shp2pgsql-gui.c 5983 2010-09-19 11:27:05Z mcayland $
  *
  * PostGIS - Spatial Types for PostgreSQL
  * http://postgis.refractions.net
@@ -17,10 +17,13 @@
 #include <stdlib.h>
 #include <string.h>
 #include <gtk/gtk.h>
+#ifdef MAC_INTEGRATION
+#include <ige-mac-integration.h>
+#endif
 #include "libpq-fe.h"
 #include "shp2pgsql-core.h"
 
-#define GUI_RCSID "shp2pgsql-gui $Revision: 5181 $"
+#define GUI_RCSID "shp2pgsql-gui $Revision: 5983 $"
 
 /*
 ** Global variables for GUI only
@@ -71,14 +74,18 @@ static void pgui_create_options_dialogue(void);
 static void
 pgui_log_va(const char *fmt, va_list ap)
 {
+	GtkTextIter iter;
 	char *msg;
 
 	if (!lw_vasprintf (&msg, fmt, ap)) return;
 
-	gtk_text_buffer_insert_at_cursor(textbuffer_log, msg, -1);
-	gtk_text_buffer_insert_at_cursor(textbuffer_log, "\n", -1);
-	gtk_text_view_scroll_mark_onscreen(GTK_TEXT_VIEW(textview_log), gtk_text_buffer_get_insert(textbuffer_log) );
+	/* Append text to the end of the text area, scrolling if required to make it visible */
+	gtk_text_buffer_get_end_iter(textbuffer_log, &iter);
+	gtk_text_buffer_insert(textbuffer_log, &iter, msg, -1);
+	gtk_text_buffer_insert(textbuffer_log, &iter, "\n", -1);
 
+	gtk_text_view_scroll_to_iter(GTK_TEXT_VIEW(textview_log), &iter, 0.0, TRUE, 0.0, 1.0);
+	
 	/* Allow GTK to process events */
 	while (gtk_events_pending())
 		gtk_main_iteration();
@@ -121,8 +128,8 @@ pgui_raise_error_dialogue(void)
 
 	label = gtk_label_new(pgui_errmsg);
 	dialog = gtk_dialog_new_with_buttons("Error", GTK_WINDOW(window_main),
-	                                     GTK_DIALOG_MODAL & GTK_DIALOG_NO_SEPARATOR & GTK_DIALOG_DESTROY_WITH_PARENT,
-	                                     GTK_STOCK_OK, GTK_RESPONSE_OK, NULL);
+										 GTK_DIALOG_MODAL & GTK_DIALOG_NO_SEPARATOR & GTK_DIALOG_DESTROY_WITH_PARENT,
+										 GTK_STOCK_OK, GTK_RESPONSE_OK, NULL);
 	gtk_dialog_set_has_separator ( GTK_DIALOG(dialog), FALSE );
 	gtk_container_set_border_width (GTK_CONTAINER(dialog), 5);
 	gtk_container_set_border_width (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox), 15);
@@ -193,9 +200,13 @@ pgui_set_config_from_options_ui()
 	else
 		config->createindex = 0;
 
-	/* Read the .shp file, don't ignore it */
+	/* Don't read the .shp file, just the .dbf */
 	if ( dbfonly )
+	{
 		config->readshape = 0;
+		/* There will be no spatial column so don't create a spatial index */
+		config->createindex = 0; 
+	}
 	else
 		config->readshape = 1;
 
@@ -432,41 +443,56 @@ pgui_read_connection(void)
 	const char *pg_pass = gtk_entry_get_text(GTK_ENTRY(entry_pg_pass));
 	const char *pg_db = gtk_entry_get_text(GTK_ENTRY(entry_pg_db));
 	char *connection_string = NULL;
+	char *escape_pg_pass = NULL;
 
-	if ( ! pg_host || strlen(pg_host) == 0 )
+	stringbuffer_t *sb = stringbuffer_create();
+
+	/* Read the host */
+	if ( pg_host && strlen(pg_host) > 0 )
 	{
-		pgui_seterr("Fill in the server host.");
-		return NULL;
+		vasbappend(sb, "host=%s ", pg_host);
 	}
-	if ( ! pg_port || strlen(pg_port) == 0 )
+
+	/* Read the port */
+	if ( pg_port && strlen(pg_port) > 0 )
+	{	
+		if ( ! atoi(pg_port) )
+		{
+			pgui_seterr("Server port must be a number.");
+			stringbuffer_destroy(sb);
+			return NULL;
+		}
+		vasbappend(sb, "port=%s ", pg_port);
+	}
+
+	/* Read the user name */
+	if ( pg_user && strlen(pg_user) > 0 )
 	{
-		pgui_seterr("Fill in the server port.");
-		return NULL;
+		vasbappend(sb, "user=%s ", pg_user);
 	}
-	if ( ! pg_user || strlen(pg_user) == 0 )
+
+	/* Read the database name */
+	if ( pg_db && strlen(pg_db) > 0 )
 	{
-		pgui_seterr("Fill in the user name.");
-		return NULL;
+		vasbappend(sb, "dbname=%s ", pg_db);
 	}
-	if ( ! pg_db || strlen(pg_db) == 0 )
+
+	/* Read the password */
+	if ( pg_pass && strlen(pg_pass) > 0 )
 	{
-		pgui_seterr("Fill in the database name.");
-		return NULL;
+		/* Escape the password in case it contains any special characters */
+		escape_pg_pass = escape_connection_string((char *)pg_pass);
+		vasbappend(sb, "password='%s' ", escape_pg_pass);
+
+		/* Free the escaped version */
+		if (escape_pg_pass != pg_pass)
+			free(escape_pg_pass);
 	}
-	if ( ! atoi(pg_port) )
-	{
-		pgui_seterr("Server port must be a number.");
-		return NULL;
-	}
-	if ( ! lw_asprintf(&connection_string, "user=%s password=%s port=%s host=%s dbname=%s", pg_user, pg_pass, pg_port, pg_host, pg_db) )
-	{
-		return NULL;
-	}
-	if ( connection_string )
-	{
-		return connection_string;
-	}
-	return NULL;
+
+	/* Return the connection string */
+	connection_string = strdup(stringbuffer_getstring(sb));
+	stringbuffer_destroy(sb);	
+	return connection_string;
 }
 
 static void
@@ -484,11 +510,14 @@ pgui_sanitize_connection_string(char *connection_string)
 	char *ptr = strstr(connection_string, "password");
 	if ( ptr )
 	{
-		ptr += 9;
-		while ( *ptr != ' ' && *ptr != '\0' )
+		ptr += 10;
+		while ( *ptr != '\'' && *ptr != '\0' )
 		{
-			*ptr = '*';
-			ptr++;
+			/* If we find a \, hide both it and the next character */
+			if ( *ptr == '\\' )
+				*ptr++ = '*';
+		
+			*ptr++ = '*';
 		}
 	}
 	return;
@@ -603,7 +632,7 @@ pgui_action_import(GtkWidget *widget, gpointer data)
 	char *connection_string = NULL;
 	char *connection_sanitized = NULL;
 	char *dest_string = NULL;
-	int ret, i = 0;
+	int ret, record_count = 0, i = 0, progress_bar_update_frequency = 1;
 	char *header, *footer, *record;
 	PGresult *result;
 
@@ -722,8 +751,13 @@ pgui_action_import(GtkWidget *widget, gpointer data)
 	/* Main loop: iterate through all of the records and send them to stdout */
 	pgui_logf("Importing shapefile (%d records)...", ShpLoaderGetRecordCount(state));
 
+    /* Number of records in this file */
+    record_count = ShpLoaderGetRecordCount(state);
+    /* Assume a progress bar 1000 pixels across, how we only want to update it every N records */
+    progress_bar_update_frequency = 1 + record_count / 1000;
+    
 	import_running = TRUE;
-	for (i = 0; i < ShpLoaderGetRecordCount(state) && import_running; i++)
+	for (i = 0; (i < record_count) && import_running; i++)
 	{
 		ret = ShpLoaderGenerateSQLRowStatement(state, i, &record);
 
@@ -775,7 +809,10 @@ pgui_action_import(GtkWidget *widget, gpointer data)
 		}
 
 		/* Update the progress bar */
-		gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(progress), (float)i / ShpLoaderGetRecordCount(state));
+        if( i % progress_bar_update_frequency == 0 )
+        {
+		    gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(progress), (float)i / record_count);
+	    }
 
 		/* Allow GTK events to get a look in */
 		while (gtk_events_pending())
@@ -985,6 +1022,10 @@ pgui_create_main_window(const SHPCONNECTIONCONFIG *conn)
 	GtkWidget *hbox_buttons, *button_options, *button_import, *button_cancel, *button_about;
 	/* Log section */
 	GtkWidget *scrolledwindow_log;
+#ifdef MAC_INTEGRATION
+	/* MacOSX Command-Quit menu item */
+	GtkWidget *menu_item_quit;
+#endif
 
 	/* create the main, top level, window */
 	window_main = gtk_window_new (GTK_WINDOW_TOPLEVEL);
@@ -1012,10 +1053,17 @@ pgui_create_main_window(const SHPCONNECTIONCONFIG *conn)
 	file_chooser_dialog_shape = gtk_file_chooser_dialog_new( "Select a Shape File", GTK_WINDOW (window_main), GTK_FILE_CHOOSER_ACTION_OPEN, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL);
 	file_chooser_button_shape = gtk_file_chooser_button_new_with_dialog( file_chooser_dialog_shape );
 	gtk_container_set_border_width (GTK_CONTAINER (file_chooser_button_shape), 8);
+	/* Filter for .shp files */
 	file_filter_shape = gtk_file_filter_new();
 	gtk_file_filter_add_pattern(GTK_FILE_FILTER(file_filter_shape), "*.shp");
-	gtk_file_filter_set_name(GTK_FILE_FILTER(file_filter_shape), "Shapefiles (*.shp)");
+	gtk_file_filter_set_name(GTK_FILE_FILTER(file_filter_shape), "Shape Files (*.shp)");
 	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(file_chooser_button_shape), file_filter_shape);
+	/* Filter for .dbf files */
+	file_filter_shape = gtk_file_filter_new();
+	gtk_file_filter_add_pattern(GTK_FILE_FILTER(file_filter_shape), "*.dbf");
+	gtk_file_filter_set_name(GTK_FILE_FILTER(file_filter_shape), "DBF Files (*.dbf)");
+	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(file_chooser_button_shape), file_filter_shape);
+	
 	gtk_container_add (GTK_CONTAINER (frame_shape), file_chooser_button_shape);
 	g_signal_connect (G_OBJECT (file_chooser_button_shape), "file-set", G_CALLBACK (pgui_action_shape_file_set), NULL);
 
@@ -1158,6 +1206,15 @@ pgui_create_main_window(const SHPCONNECTIONCONFIG *conn)
 	gtk_container_add (GTK_CONTAINER (scrolledwindow_log), textview_log);
 	gtk_container_add (GTK_CONTAINER (frame_log), scrolledwindow_log);
 
+#ifdef MAC_INTEGRATION
+	/*
+	** OS/X menu integration for command-Q
+	*/
+	menu_item_quit = gtk_menu_item_new();
+	g_signal_connect (G_OBJECT (menu_item_quit), "activate", G_CALLBACK (pgui_action_cancel), NULL);
+	ige_mac_menu_set_quit_menu_item (GTK_MENU_ITEM (menu_item_quit));
+#endif 
+
 	/*
 	** Main window
 	*/
@@ -1197,34 +1254,42 @@ main(int argc, char *argv[])
 {
 	char c;
 
-	/* Parse command line options and set configuration */
+	/* Set default configuration */
 	config = malloc(sizeof(SHPLOADERCONFIG));
 	set_config_defaults(config);
 
 	/* Here we override any defaults for the GUI */
 	config->createindex = 1;
 
+	/* Prepare our shape connection */
 	conn = malloc(sizeof(SHPCONNECTIONCONFIG));
 	memset(conn, 0, sizeof(SHPCONNECTIONCONFIG));
 
+    /* Read any environment values */
+    conn->port = getenv("PGPORT");
+    conn->username = getenv("PGUSER");
+    conn->database = getenv("PGDATABASE");
+    conn->host = getenv("PGHOST");
+
+	/* Over-ride the environment with any command line options */
 	while ((c = pgis_getopt(argc, argv, "U:p:W:d:h:")) != -1)
 	{
 		switch (c)
 		{
 		case 'U':
-			conn->username = optarg;
+			conn->username = pgis_optarg;
 			break;
 		case 'p':
-			conn->port = optarg;
+			conn->port = pgis_optarg;
 			break;
 		case 'W':
-			conn->password = optarg;
+			conn->password = pgis_optarg;
 			break;
 		case 'd':
-			conn->database = optarg;
+			conn->database = pgis_optarg;
 			break;
 		case 'h':
-			conn->host = optarg;
+			conn->host = pgis_optarg;
 			break;
 		default:
 			usage();
