@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: lwgeom_accum.c 5397 2010-03-10 15:38:12Z pramsey $
+ * $Id: lwgeom_accum.c 7197 2011-05-19 02:32:27Z pramsey $
  *
  * PostGIS - Spatial Types for PostgreSQL
  * http://postgis.refractions.net
@@ -169,9 +169,7 @@ pgis_accum_finalfn(pgis_abs *p, MemoryContext mctx, FunctionCallInfo fcinfo)
 #if POSTGIS_PGSQL_VERSION < 84
 	result = makeMdArrayResult(state, 1, dims, lbs, mctx);
 #else
-	/* Release working state if regular aggregate, but not if window agg */
-	result = makeMdArrayResult(state, 1, dims, lbs, mctx,
-	                           IsA(fcinfo->context, AggState));
+	result = makeMdArrayResult(state, 1, dims, lbs, mctx, false);
 #endif
 	return result;
 }
@@ -305,7 +303,12 @@ PGISDirectFunctionCall1(PGFunction func, Datum arg1)
 	FunctionCallInfoData fcinfo;
 	Datum           result;
 
+#if POSTGIS_PGSQL_VERSION > 90
+  /* PgSLQ 9.1 requires five arcuments to InitFunctionCallInfoData */
+	InitFunctionCallInfoData(fcinfo, NULL, 1, InvalidOid, NULL, NULL);
+#else
 	InitFunctionCallInfoData(fcinfo, NULL, 1, NULL, NULL);
+#endif
 
 	fcinfo.arg[0] = arg1;
 	fcinfo.argnull[0] = false;
