@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: lwgeom_estimate.c 4168 2009-06-11 16:44:03Z pramsey $
+ * $Id: lwgeom_estimate.c 5181 2010-02-01 17:35:55Z pramsey $
  *
  * PostGIS - Spatial Types for PostgreSQL
  * http://postgis.refractions.net
@@ -33,7 +33,7 @@
 #include <errno.h>
 #include <ctype.h>
 
-/*
+/**
  * 	Assign a number to the postgis statistics kind
  *
  * 	tgl suggested:
@@ -86,17 +86,17 @@ static float8 estimate_selectivity(BOX2DFLOAT4 *box, GEOM_STATS *geomstats);
 #define SHOW_DIGS_DOUBLE 15
 #define MAX_DIGS_DOUBLE (SHOW_DIGS_DOUBLE + 6 + 1 + 3 +1)
 
-/*
+/**
  * Default geometry selectivity factor
  */
 #define DEFAULT_GEOMETRY_SEL 0.000005
 
-/*
+/**
  * Default geometry join selectivity factor
  */
 #define DEFAULT_GEOMETRY_JOINSEL 0.000005
 
-/*
+/**
  * Define this to actually DO join selectivity
  * (as contrary to just return the default JOINSEL value)
  * Note that this is only possible when compiling postgis
@@ -111,7 +111,7 @@ Datum LWGEOM_analyze(PG_FUNCTION_ARGS);
 
 
 #if ! REALLY_DO_JOINSEL
-/*
+/**
  * JOIN selectivity in the GiST && operator
  * for all PG versions
  */
@@ -131,7 +131,7 @@ int calculate_column_intersection(BOX2DFLOAT4 *search_box, GEOM_STATS *geomstats
 int
 calculate_column_intersection(BOX2DFLOAT4 *search_box, GEOM_STATS *geomstats1, GEOM_STATS *geomstats2)
 {
-	/*
+	/**
 	* Calculate the intersection of two columns from their geomstats extents - return true
 	* if a valid intersection was found, false if there is no overlap
 	*/
@@ -154,7 +154,7 @@ calculate_column_intersection(BOX2DFLOAT4 *search_box, GEOM_STATS *geomstats1, G
 	return -1;
 }
 
-/*
+/**
 * JOIN selectivity in the GiST && operator
 * for all PG versions
 */
@@ -186,7 +186,7 @@ Datum LWGEOM_gist_joinsel(PG_FUNCTION_ARGS)
 	BOX2DFLOAT4 search_box;
 
 
-	/*
+	/**
 	* Join selectivity algorithm. To calculation the selectivity we
 	* calculate the intersection of the two column sample extents,
 	* sum the results, and then multiply by two since for each
@@ -237,8 +237,10 @@ Datum LWGEOM_gist_joinsel(PG_FUNCTION_ARGS)
 
 
 
-	if ( ! get_attstatsslot(stats1_tuple, 0, 0,
-	                        STATISTIC_KIND_GEOMETRY, InvalidOid, NULL, NULL,
+	if ( ! get_attstatsslot(stats1_tuple, 0, 0, STATISTIC_KIND_GEOMETRY, InvalidOid, NULL, NULL,
+#if POSTGIS_PGSQL_VERSION >= 85
+	                        NULL,
+#endif
 	                        (float4 **)gs1ptr, &geomstats1_nvalues) )
 	{
 		POSTGIS_DEBUG(3, " STATISTIC_KIND_GEOMETRY stats not found - returning default geometry join selectivity");
@@ -261,8 +263,10 @@ Datum LWGEOM_gist_joinsel(PG_FUNCTION_ARGS)
 	}
 
 
-	if ( ! get_attstatsslot(stats2_tuple, 0, 0,
-	                        STATISTIC_KIND_GEOMETRY, InvalidOid, NULL, NULL,
+	if ( ! get_attstatsslot(stats2_tuple, 0, 0, STATISTIC_KIND_GEOMETRY, InvalidOid, NULL, NULL,
+#if POSTGIS_PGSQL_VERSION >= 85
+	                        NULL,
+#endif
 	                        (float4 **)gs2ptr, &geomstats2_nvalues) )
 	{
 		POSTGIS_DEBUG(3, " STATISTIC_KIND_GEOMETRY stats not found - returning default geometry join selectivity");
@@ -275,7 +279,7 @@ Datum LWGEOM_gist_joinsel(PG_FUNCTION_ARGS)
 	}
 
 
-	/*
+	/**
 	* Setup the search box - this is the intersection of the two column
 	* extents.
 	*/
@@ -374,15 +378,16 @@ Datum LWGEOM_gist_joinsel(PG_FUNCTION_ARGS)
 /**************************** FROM POSTGIS ****************/
 
 
-/*
+/**
  * This function returns an estimate of the selectivity
  * of a search_box looking at data in the GEOM_STATS
  * structure.
- *
- * TODO: handle box dimension collapses (probably should be handled
- * by the statistic generator, avoiding GEOM_STATS with collapsed
- * dimensions)
- */
+ * */
+/**
+* TODO: handle box dimension collapses (probably should be handled
+* by the statistic generator, avoiding GEOM_STATS with collapsed
+* dimensions)
+*/
 static float8
 estimate_selectivity(BOX2DFLOAT4 *box, GEOM_STATS *geomstats)
 {
@@ -561,7 +566,7 @@ estimate_selectivity(BOX2DFLOAT4 *box, GEOM_STATS *geomstats)
 	 * by both searc_box and feature. We should then
 	 * divide our value by the number of cells in the virtual
 	 * 'intersection' between average feature cell occupation
-	 * and occupation of the search_box. This is as 
+	 * and occupation of the search_box. This is as
 	 * fuzzy as you understand it :)
 	 *
 	 * Consistency check: whenever the number of cells is
@@ -600,7 +605,7 @@ estimate_selectivity(BOX2DFLOAT4 *box, GEOM_STATS *geomstats)
 	return selectivity;
 }
 
-/*
+/**
  * This function should return an estimation of the number of
  * rows returned by a query involving an overlap check
  * ( it's the restrict function for the && operator )
@@ -674,7 +679,7 @@ Datum LWGEOM_gist_sel(PG_FUNCTION_ARGS)
 	/*
 	 * We are working on two constants..
 	 * TODO: check if expression is true,
-	 *       returned set would be either 
+	 *       returned set would be either
 	 *       the whole or none.
 	 */
 	if ( ! IsA(self, Var) )
@@ -713,8 +718,10 @@ Datum LWGEOM_gist_sel(PG_FUNCTION_ARGS)
 	}
 
 
-	if ( ! get_attstatsslot(stats_tuple, 0, 0,
-	                        STATISTIC_KIND_GEOMETRY, InvalidOid, NULL, NULL,
+	if ( ! get_attstatsslot(stats_tuple, 0, 0, STATISTIC_KIND_GEOMETRY, InvalidOid, NULL, NULL,
+#if POSTGIS_PGSQL_VERSION >= 85
+	                        NULL,
+#endif
 	                        (float4 **)gsptr, &geomstats_nvalues) )
 	{
 		POSTGIS_DEBUG(3, " STATISTIC_KIND_GEOMETRY stats not found - returning default geometry selectivity");
@@ -749,7 +756,7 @@ Datum LWGEOM_gist_sel(PG_FUNCTION_ARGS)
 }
 
 
-/*
+/**
  * This function is called by the analyze function iff
  * the geometry_analyze() function give it its pointer
  * (this is always the case so far).
@@ -895,13 +902,13 @@ compute_geometry_stats(VacAttrStats *stats, AnalyzeAttrFetchFunc fetchfunc,
 			                                  box.ymin);
 		}
 
-		/* TODO: ask if we need geom or bvol size for stawidth */
+		/** TODO: ask if we need geom or bvol size for stawidth */
 		total_width += geom->size;
 		total_boxes_area += (box.xmax-box.xmin)*(box.ymax-box.ymin);
 
 #if USE_STANDARD_DEVIATION
 		/*
-		 * Add bvol coordinates to sum for standard deviation 
+		 * Add bvol coordinates to sum for standard deviation
 		 * computation.
 		 */
 		sumLOWx += box.xmin;
@@ -1014,7 +1021,7 @@ compute_geometry_stats(VacAttrStats *stats, AnalyzeAttrFetchFunc fetchfunc,
 
 	/*
 	 * Set histogram extent as the intersection between
-	 * standard deviation based histogram extent 
+	 * standard deviation based histogram extent
 	 * and computed sample extent after removal of
 	 * hard deviants (there might be no hard deviants).
 	 */
@@ -1104,7 +1111,7 @@ compute_geometry_stats(VacAttrStats *stats, AnalyzeAttrFetchFunc fetchfunc,
 	geomstats->rows = rows;
 
 	/* Initialize all values to 0 */
-	for (i=0;i<histocells; i++) geomstats->value[i] = 0;
+	for (i=0; i<histocells; i++) geomstats->value[i] = 0;
 
 	cell_width = geow/cols;
 	cell_height = geoh/rows;
@@ -1179,7 +1186,7 @@ compute_geometry_stats(VacAttrStats *stats, AnalyzeAttrFetchFunc fetchfunc,
 
 		/*
 		 * before adding to the total cells
-		 * we could decide if we really 
+		 * we could decide if we really
 		 * want this feature to count
 		 */
 		total_boxes_cells += numcells;
@@ -1199,7 +1206,7 @@ compute_geometry_stats(VacAttrStats *stats, AnalyzeAttrFetchFunc fetchfunc,
 		return;
 	}
 
-	/* what about null features (TODO) ? */
+	/** TODO: what about null features (TODO) */
 	geomstats->avgFeatureCells = (float4)total_boxes_cells/examinedsamples;
 
 	POSTGIS_DEBUGF(3, " histo: total_boxes_cells: %d", total_boxes_cells);
@@ -1230,7 +1237,7 @@ compute_geometry_stats(VacAttrStats *stats, AnalyzeAttrFetchFunc fetchfunc,
 
 
 	/*
-	 * Write the statistics data 
+	 * Write the statistics data
 	 */
 	stats->stakind[0] = STATISTIC_KIND_GEOMETRY;
 	stats->staop[0] = InvalidOid;
@@ -1252,7 +1259,7 @@ compute_geometry_stats(VacAttrStats *stats, AnalyzeAttrFetchFunc fetchfunc,
 	stats->stats_valid = true;
 }
 
-/*
+/**
  * This function will be called when the ANALYZE command is run
  * on a column of the "geometry" type.
  *
@@ -1313,7 +1320,7 @@ Datum LWGEOM_analyze(PG_FUNCTION_ARGS)
 	PG_RETURN_BOOL(true);
 }
 
-/*
+/**
  * Return the estimated extent of the table
  * looking at gathered statistics (or NULL if
  * no statistics have been gathered).
