@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: lwgeom_geojson.c 4168 2009-06-11 16:44:03Z pramsey $
+ * $Id: lwgeom_geojson.c 5310 2010-02-22 22:03:25Z colivier $
  *
  * PostGIS - Spatial Types for PostgreSQL
  * http://postgis.refractions.net
@@ -16,15 +16,13 @@
 **********************************************************************/
 
 #include "postgres.h"
-#include "executor/spi.h"
 
 #include "lwgeom_pg.h"
 #include "liblwgeom.h"
+#include "lwgeom_export.h"
 
 Datum LWGEOM_asGeoJson(PG_FUNCTION_ARGS);
 
-char *geometry_to_geojson(uchar *srl, char *srs, bool has_bbox, int precision);
-
 static char *asgeojson_point(LWPOINT *point, char *srs, BOX3D *bbox, int precision);
 static char *asgeojson_line(LWLINE *line, char *srs, BOX3D *bbox, int precision);
 static char *asgeojson_poly(LWPOLY *poly, char *srs, BOX3D *bbox, int precision);
@@ -37,11 +35,6 @@ static size_t asgeojson_inspected_buf(LWGEOM_INSPECTED *insp, char *output, BOX3
 
 static size_t pointArray_to_geojson(POINTARRAY *pa, char *buf, int precision);
 static size_t pointArray_geojson_size(POINTARRAY *pa, int precision);
-static char *getSRSbySRID(int SRID, bool short_crs);
-
-#define SHOW_DIGS_DOUBLE 15
-#define MAX_DOUBLE_PRECISION 15
-#define MAX_DIGS_DOUBLE (SHOW_DIGS_DOUBLE + 2) /* +2 mean add dot and sign */
 
 
 /**
@@ -123,8 +116,7 @@ Datum LWGEOM_asGeoJson(PG_FUNCTION_ARGS)
 }
 
 
-
-/*
+/**
  * Takes a GEOMETRY and returns a GeoJson representation
  */
 char *
@@ -199,6 +191,7 @@ geometry_to_geojson(uchar *geom, char *srs, bool has_bbox, int precision)
 }
 
 
+
 /**
  * Handle SRS
  */
@@ -226,13 +219,13 @@ asgeojson_srs_buf(char *output, char *srs)
 }
 
 
+
 /**
  * Handle Bbox
  */
 static size_t
 asgeojson_bbox_size(bool hasz, int precision)
 {
-
 	int size;
 
 	if (!hasz)
@@ -664,7 +657,7 @@ asgeojson_collection_size(LWGEOM_INSPECTED *insp, char *srs, BOX3D *bbox, int pr
 	{
 		subgeom = lwgeom_getsubgeometry_inspected(insp, i);
 		subinsp = lwgeom_inspect(subgeom);
-		size += asgeojson_inspected_size(subinsp, bbox, precision);
+		size += asgeojson_inspected_size(subinsp, NULL, precision);
 		lwinspected_release(subinsp);
 	}
 	size += sizeof(",") * i;
@@ -691,7 +684,7 @@ asgeojson_collection_buf(LWGEOM_INSPECTED *insp, char *srs, char *output, BOX3D
 		if (i) ptr += sprintf(ptr, ",");
 		subgeom = lwgeom_getsubgeometry_inspected(insp, i);
 		subinsp = lwgeom_inspect(subgeom);
-		ptr += asgeojson_inspected_buf(subinsp, ptr, bbox, precision);
+		ptr += asgeojson_inspected_buf(subinsp, ptr, NULL, precision);
 		lwinspected_release(subinsp);
 	}
 
@@ -773,15 +766,6 @@ asgeojson_inspected_buf(LWGEOM_INSPECTED *insp, char *output, BOX3D *bbox, int p
 	int type = lwgeom_getType(insp->serialized_form[0]);
 	char *ptr=output;
 
-	/* Compute 3D BBOX on the sub geometry */
-	if ( bbox )
-	{
-		lwfree(bbox);
-		bbox = NULL;
-		bbox = compute_serialized_box3d(
-		           lwgeom_getsubgeometry(insp->serialized_form, 0));
-	}
-
 	switch (type)
 	{
 	case POINTTYPE:
@@ -840,10 +824,19 @@ pointArray_to_geojson(POINTARRAY *pa, char *output, int precision)
 		{
 			POINT2D pt;
 			getPoint2d_p(pa, i, &pt);
-			sprintf(x, "%.*f", precision, pt.x);
+
+			if (fabs(pt.x) < MAX_DOUBLE)
+				sprintf(x, "%.*f", precision, pt.x);
+			else
+				sprintf(x, "%g", pt.x);
 			trim_trailing_zeros(x);
-			sprintf(y, "%.*f", precision, pt.y);
+
+			if (fabs(pt.y) < MAX_DOUBLE)
+				sprintf(y, "%.*f", precision, pt.y);
+			else
+				sprintf(y, "%g", pt.y);
 			trim_trailing_zeros(y);
+
 			if ( i ) ptr += sprintf(ptr, ",");
 			ptr += sprintf(ptr, "[%s,%s]", x, y);
 		}
@@ -854,12 +847,25 @@ pointArray_to_geojson(POINTARRAY *pa, char *output, int precision)
 		{
 			POINT4D pt;
 			getPoint4d_p(pa, i, &pt);
-			sprintf(x, "%.*f", precision, pt.x);
+
+			if (fabs(pt.x) < MAX_DOUBLE)
+				sprintf(x, "%.*f", precision, pt.x);
+			else
+				sprintf(x, "%g", pt.x);
 			trim_trailing_zeros(x);
-			sprintf(y, "%.*f", precision, pt.y);
+
+			if (fabs(pt.y) < MAX_DOUBLE)
+				sprintf(y, "%.*f", precision, pt.y);
+			else
+				sprintf(y, "%g", pt.y);
 			trim_trailing_zeros(y);
-			sprintf(z, "%.*f", precision, pt.z);
+
+			if (fabs(pt.z) < MAX_DOUBLE)
+				sprintf(z, "%.*f", precision, pt.z);
+			else
+				sprintf(z, "%g", pt.z);
 			trim_trailing_zeros(z);
+
 			if ( i ) ptr += sprintf(ptr, ",");
 			ptr += sprintf(ptr, "[%s,%s,%s]", x, y, z);
 		}
@@ -869,67 +875,6 @@ pointArray_to_geojson(POINTARRAY *pa, char *output, int precision)
 }
 
 
-/*
- * Common geojson routines
- */
-
-static char *
-getSRSbySRID(int SRID, bool short_crs)
-{
-	char query[256];
-	char *srs, *srscopy;
-	int size, err;
-
-	if (SPI_OK_CONNECT != SPI_connect ())
-	{
-		elog(NOTICE, "getSRSbySRID: could not connect to SPI manager");
-		SPI_finish();
-		return NULL;
-	}
-
-	if (short_crs)
-		sprintf(query, "SELECT auth_name||':'||auth_srid \
-		        FROM spatial_ref_sys WHERE srid='%d'", SRID);
-	else
-		sprintf(query, "SELECT 'urn:ogc:def:crs:'||auth_name||':'||auth_srid \
-		        FROM spatial_ref_sys WHERE srid='%d'", SRID);
-
-	err = SPI_exec(query, 1);
-	if ( err < 0 )
-	{
-		elog(NOTICE, "getSRSbySRID: error executing query %d", err);
-		SPI_finish();
-		return NULL;
-	}
-
-	/* no entry in spatial_ref_sys */
-	if (SPI_processed <= 0)
-	{
-		SPI_finish();
-		return NULL;
-	}
-
-	/* get result  */
-	srs = SPI_getvalue(SPI_tuptable->vals[0], SPI_tuptable->tupdesc, 1);
-
-	/* NULL result */
-	if ( ! srs )
-	{
-		SPI_finish();
-		return NULL;
-	}
-
-	/* copy result to upper executor context */
-	size = strlen(srs)+1;
-	srscopy = SPI_palloc(size);
-	memcpy(srscopy, srs, size);
-
-	/* disconnect from SPI */
-	SPI_finish();
-
-	return srscopy;
-}
-
 
 /**
  * Returns maximum size of rendered pointarray in bytes.
