@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: lwgeom_geos.c 5181 2010-02-01 17:35:55Z pramsey $
+ * $Id: lwgeom_geos.c 7344 2011-06-09 13:59:42Z pramsey $
  *
  * PostGIS - Spatial Types for PostgreSQL
  * http://postgis.refractions.net
@@ -16,6 +16,7 @@
 #include "lwgeom_geos.h"
 #include "lwgeom_rtree.h"
 #include "lwgeom_geos_prepared.h"
+#include "lwgeom_functions_analytic.h"
 
 #include <string.h>
 
@@ -69,13 +70,6 @@ Datum pgis_union_geometry_array_old(PG_FUNCTION_ARGS);
 Datum pgis_union_geometry_array(PG_FUNCTION_ARGS);
 
 
-/** @todo TODO: move these to a lwgeom_functions_analytic.h
-	*/
-int point_in_polygon_rtree(RTREE_NODE **root, int ringCount, LWPOINT *point);
-int point_in_multipolygon_rtree(RTREE_NODE **root, int polyCount, int ringCount, LWPOINT *point);
-int point_in_polygon(LWPOLY *polygon, LWPOINT *point);
-int point_in_multipolygon(LWMPOLY *mpolygon, LWPOINT *pont);
-
 /*
 ** Prototypes end
 */
@@ -1364,13 +1358,29 @@ Datum isvalid(PG_FUNCTION_ARGS)
 	LWGEOM *lwgeom;
 	bool result;
 	GEOSGeom g1;
+#if POSTGIS_GEOS_VERSION < 33
+  BOX2DFLOAT4 box1;
+#endif
 
 	PROFSTART(PROF_QRUN);
 
 	geom1 = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
 
-	initGEOS(lwnotice, lwnotice);
+#if POSTGIS_GEOS_VERSION < 33
+  /* Short circuit and return FALSE if we have infinite coordinates */
+  /* GEOS 3.3+ is supposed to  handle this stuff OK */
+	if ( getbox2d_p(SERIALIZED_FORM(geom1), &box1) )	
+	{
+		if ( isinf(box1.xmax) || isinf(box1.ymax) || isinf(box1.xmin) || isinf(box1.ymin) || 
+		     isnan(box1.xmax) || isnan(box1.ymax) || isnan(box1.xmin) || isnan(box1.ymin)  )
+		{
+      lwnotice("Geometry contains an Inf or NaN coordinate");
+			PG_RETURN_BOOL(FALSE);
+		}
+	}
+#endif
 
+	initGEOS(lwnotice, lwnotice);
 	PROFSTART(PROF_P2G1);
 
 	lwgeom = lwgeom_deserialize(SERIALIZED_FORM(geom1));
@@ -1378,6 +1388,8 @@ Datum isvalid(PG_FUNCTION_ARGS)
 	{
 		lwerror("unable to deserialize input");
 	}
+	
+	
 	g1 = LWGEOM2GEOS(lwgeom);
 	if ( ! g1 )
 	{
@@ -1419,9 +1431,31 @@ Datum isvalidreason(PG_FUNCTION_ARGS)
 	int len = 0;
 	char *result = NULL;
 	const GEOSGeometry *g1 = NULL;
+#if POSTGIS_GEOS_VERSION < 33
+  BOX2DFLOAT4 box;
+#endif
 
 	geom = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
 
+#if POSTGIS_GEOS_VERSION < 33
+  /* Short circuit and return if we have infinite coordinates */
+  /* GEOS 3.3+ is supposed to  handle this stuff OK */
+	if ( getbox2d_p(SERIALIZED_FORM(geom), &box) )	
+	{
+		if ( isinf(box.xmax) || isinf(box.ymax) || isinf(box.xmin) || isinf(box.ymin) || 
+		     isnan(box.xmax) || isnan(box.ymax) || isnan(box.xmin) || isnan(box.ymin)  )
+		{
+			const char *rsn = "Geometry contains an Inf or NaN coordinate";
+    	len = strlen(rsn);
+    	result = palloc(VARHDRSZ + len);
+    	SET_VARSIZE(result, VARHDRSZ + len);
+    	memcpy(VARDATA(result), rsn, len);
+    	PG_FREE_IF_COPY(geom, 0);
+    	PG_RETURN_POINTER(result);			
+		}
+	}
+#endif
+
 	initGEOS(lwnotice, lwnotice);
 
 	g1 = (GEOSGeometry *)POSTGIS2GEOS(geom);
@@ -1585,7 +1619,7 @@ Datum contains(PG_FUNCTION_ARGS)
 
 		if ( poly_cache->ringIndices )
 		{
-			result = point_in_multipolygon_rtree(poly_cache->ringIndices, poly_cache->polyCount, poly_cache->ringCount, point);
+			result = point_in_multipolygon_rtree(poly_cache->ringIndices, poly_cache->polyCount, poly_cache->ringCounts, point);
 		}
 		else if ( type1 == POLYGONTYPE )
 		{
@@ -1785,7 +1819,7 @@ Datum covers(PG_FUNCTION_ARGS)
 
 		if ( poly_cache->ringIndices )
 		{
-			result = point_in_multipolygon_rtree(poly_cache->ringIndices, poly_cache->polyCount, poly_cache->ringCount, point);
+			result = point_in_multipolygon_rtree(poly_cache->ringIndices, poly_cache->polyCount, poly_cache->ringCounts, point);
 		}
 		else if ( type1 == POLYGONTYPE )
 		{
@@ -1917,7 +1951,7 @@ Datum within(PG_FUNCTION_ARGS)
 
 		if ( poly_cache->ringIndices )
 		{
-			result = point_in_multipolygon_rtree(poly_cache->ringIndices, poly_cache->polyCount, poly_cache->ringCount, point);
+			result = point_in_multipolygon_rtree(poly_cache->ringIndices, poly_cache->polyCount, poly_cache->ringCounts, point);
 		}
 		else if ( type2 == POLYGONTYPE )
 		{
@@ -2049,7 +2083,7 @@ Datum coveredby(PG_FUNCTION_ARGS)
 
 		if ( poly_cache->ringIndices )
 		{
-			result = point_in_multipolygon_rtree(poly_cache->ringIndices, poly_cache->polyCount, poly_cache->ringCount, point);
+			result = point_in_multipolygon_rtree(poly_cache->ringIndices, poly_cache->polyCount, poly_cache->ringCounts, point);
 		}
 		else if ( type2 == POLYGONTYPE )
 		{
@@ -2254,7 +2288,7 @@ Datum intersects(PG_FUNCTION_ARGS)
 
 		if ( poly_cache->ringIndices )
 		{
-			result = point_in_multipolygon_rtree(poly_cache->ringIndices, poly_cache->polyCount, poly_cache->ringCount, point);
+			result = point_in_multipolygon_rtree(poly_cache->ringIndices, poly_cache->polyCount, poly_cache->ringCounts, point);
 		}
 		else if ( polytype == POLYGONTYPE )
 		{
@@ -2905,6 +2939,13 @@ ptarray_to_GEOSCoordSeq(POINTARRAY *pa)
 
 		POSTGIS_DEBUGF(4, "Point: %g,%g,%g", p.x, p.y, p.z);
 
+#if POSTGIS_GEOS_VERSION < 33
+    /* Make sure we don't pass any infinite values down into GEOS */
+    /* GEOS 3.3+ is supposed to  handle this stuff OK */
+    if ( isinf(p.x) || isinf(p.y) || (dims == 3 && isinf(p.z)) )
+      lwerror("Infinite coordinate value found in geometry.");
+#endif
+
 		GEOSCoordSeq_setX(sq, i, p.x);
 		GEOSCoordSeq_setY(sq, i, p.y);
 		if ( dims == 3 ) GEOSCoordSeq_setZ(sq, i, p.z);
