@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: lwgeom_gml.c 4168 2009-06-11 16:44:03Z pramsey $
+ * $Id: lwgeom_gml.c 5268 2010-02-20 19:09:14Z colivier $
  *
  * PostGIS - Spatial Types for PostgreSQL
  * http://postgis.refractions.net
@@ -17,44 +17,39 @@
 
 
 #include "postgres.h"
-#include "executor/spi.h"
 
 #include "lwgeom_pg.h"
 #include "liblwgeom.h"
+#include "lwgeom_export.h"
+
 
 Datum LWGEOM_asGML(PG_FUNCTION_ARGS);
 
-char *geometry_to_gml2(uchar *srl, char *srs, int precision);
-
 static size_t asgml2_point_size(LWPOINT *point, char *srs, int precision);
 static char *asgml2_point(LWPOINT *point, char *srs, int precision);
 static size_t asgml2_line_size(LWLINE *line, char *srs, int precision);
 static char *asgml2_line(LWLINE *line, char *srs, int precision);
 static size_t asgml2_poly_size(LWPOLY *poly, char *srs, int precision);
 static char *asgml2_poly(LWPOLY *poly, char *srs, int precision);
-static size_t asgml2_inspected_size(LWGEOM_INSPECTED *geom, char *srs, int precision);
-static char *asgml2_inspected(LWGEOM_INSPECTED *geom, char *srs, int precision);
+static size_t asgml2_multi_size(LWGEOM_INSPECTED *geom, char *srs, int precision);
+static char *asgml2_multi(LWGEOM_INSPECTED *geom, char *srs, int precision);
+static size_t asgml2_collection_size(LWGEOM_INSPECTED *geom, char *srs, int precision);
+static char *asgml2_collection(LWGEOM_INSPECTED *geom, char *srs, int precision);
 static size_t pointArray_toGML2(POINTARRAY *pa, char *buf, int precision);
 
-char *geometry_to_gml3(uchar *srl, char *srs, int precision, bool is_deegree);
-
 static size_t asgml3_point_size(LWPOINT *point, char *srs, int precision);
 static char *asgml3_point(LWPOINT *point, char *srs, int precision, bool is_deegree);
 static size_t asgml3_line_size(LWLINE *line, char *srs, int precision);
 static char *asgml3_line(LWLINE *line, char *srs, int precision, bool is_deegree);
 static size_t asgml3_poly_size(LWPOLY *poly, char *srs, int precision);
 static char *asgml3_poly(LWPOLY *poly, char *srs, int precision, bool is_deegree);
-static size_t asgml3_inspected_size(LWGEOM_INSPECTED *geom, char *srs, int precision);
-static char *asgml3_inspected(LWGEOM_INSPECTED *geom, char *srs, int precision, bool is_deegree);
+static size_t asgml3_multi_size(LWGEOM_INSPECTED *geom, char *srs, int precision);
+static char *asgml3_multi(LWGEOM_INSPECTED *geom, char *srs, int precision, bool is_deegree);
+static size_t asgml3_collection_size(LWGEOM_INSPECTED *geom, char *srs, int precision);
+static char *asgml3_collection(LWGEOM_INSPECTED *insp, char *srs, int precision, bool is_deegree);
 static size_t pointArray_toGML3(POINTARRAY *pa, char *buf, int precision, bool is_deegree);
 
 static size_t pointArray_GMLsize(POINTARRAY *pa, int precision);
-static char *getSRSbySRID(int SRID, bool short_crs);
-
-
-#define SHOW_DIGS_DOUBLE 15
-#define MAX_DOUBLE_PRECISION 15
-#define MAX_DIGS_DOUBLE (SHOW_DIGS_DOUBLE + 2) /* +2 mean add dot and sign */
 
 
 /**
@@ -159,9 +154,12 @@ geometry_to_gml2(uchar *geom, char *srs, int precision)
 	case MULTIPOINTTYPE:
 	case MULTILINETYPE:
 	case MULTIPOLYGONTYPE:
+		inspected = lwgeom_inspect(geom);
+		return asgml2_multi(inspected, srs, precision);
+
 	case COLLECTIONTYPE:
 		inspected = lwgeom_inspect(geom);
-		return asgml2_inspected(inspected, srs, precision);
+		return asgml2_collection(inspected, srs, precision);
 
 	default:
 		lwerror("geometry_to_gml2: '%s' geometry type not supported", lwgeom_typename(type));
@@ -317,7 +315,7 @@ asgml2_poly(LWPOLY *poly, char *srs, int precision)
  * Don't call this with single-geoms inspected.
  */
 static size_t
-asgml2_inspected_size(LWGEOM_INSPECTED *insp, char *srs, int precision)
+asgml2_multi_size(LWGEOM_INSPECTED *insp, char *srs, int precision)
 {
 	int i;
 	size_t size;
@@ -332,8 +330,6 @@ asgml2_inspected_size(LWGEOM_INSPECTED *insp, char *srs, int precision)
 		LWPOINT *point;
 		LWLINE *line;
 		LWPOLY *poly;
-		LWGEOM_INSPECTED *subinsp;
-		uchar *subgeom;
 
 		if ((point=lwgeom_getpoint_inspected(insp, i)))
 		{
@@ -353,13 +349,6 @@ asgml2_inspected_size(LWGEOM_INSPECTED *insp, char *srs, int precision)
 			size += asgml2_poly_size(poly, 0, precision);
 			lwpoly_release(poly);
 		}
-		else
-		{
-			subgeom = lwgeom_getsubgeometry_inspected(insp, i);
-			subinsp = lwgeom_inspect(subgeom);
-			size += asgml2_inspected_size(subinsp, 0, precision);
-			lwinspected_release(subinsp);
-		}
 	}
 
 	return size;
@@ -369,18 +358,18 @@ asgml2_inspected_size(LWGEOM_INSPECTED *insp, char *srs, int precision)
  * Don't call this with single-geoms inspected!
  */
 static size_t
-asgml2_inspected_buf(LWGEOM_INSPECTED *insp, char *srs, char *output, int precision)
+asgml2_multi_buf(LWGEOM_INSPECTED *insp, char *srs, char *output, int precision)
 {
 	int type = lwgeom_getType(insp->serialized_form[0]);
 	char *ptr, *gmltype;
 	int i;
 
 	ptr = output;
+	gmltype="";
 
-	if (type == MULTIPOINTTYPE) gmltype = "MultiPoint";
-	else if (type == MULTILINETYPE) gmltype = "MultiLineString";
+	if 	(type == MULTIPOINTTYPE)   gmltype = "MultiPoint";
+	else if (type == MULTILINETYPE)	   gmltype = "MultiLineString";
 	else if (type == MULTIPOLYGONTYPE) gmltype = "MultiPolygon";
-	else gmltype = "MultiGeometry";
 
 	/* Open outmost tag */
 	if ( srs )
@@ -397,8 +386,6 @@ asgml2_inspected_buf(LWGEOM_INSPECTED *insp, char *srs, char *output, int precis
 		LWPOINT *point;
 		LWLINE *line;
 		LWPOLY *poly;
-		LWGEOM_INSPECTED *subinsp;
-		uchar *subgeom;
 
 		if ((point=lwgeom_getpoint_inspected(insp, i)))
 		{
@@ -421,13 +408,6 @@ asgml2_inspected_buf(LWGEOM_INSPECTED *insp, char *srs, char *output, int precis
 			lwpoly_release(poly);
 			ptr += sprintf(ptr, "</gml:polygonMember>");
 		}
-		else
-		{
-			subgeom = lwgeom_getsubgeometry_inspected(insp, i);
-			subinsp = lwgeom_inspect(subgeom);
-			ptr += asgml2_inspected_buf(subinsp, 0, ptr, precision);
-			lwinspected_release(subinsp);
-		}
 	}
 
 	/* Close outmost tag */
@@ -440,17 +420,144 @@ asgml2_inspected_buf(LWGEOM_INSPECTED *insp, char *srs, char *output, int precis
  * Don't call this with single-geoms inspected!
  */
 static char *
-asgml2_inspected(LWGEOM_INSPECTED *insp, char *srs, int precision)
+asgml2_multi(LWGEOM_INSPECTED *insp, char *srs, int precision)
 {
 	char *gml;
 	size_t size;
 
-	size = asgml2_inspected_size(insp, srs, precision);
+	size = asgml2_multi_size(insp, srs, precision);
 	gml = palloc(size);
-	asgml2_inspected_buf(insp, srs, gml, precision);
+	asgml2_multi_buf(insp, srs, gml, precision);
 	return gml;
 }
 
+
+static size_t
+asgml2_collection_size(LWGEOM_INSPECTED *insp, char *srs, int precision)
+{
+	int i;
+	size_t size;
+
+	size = sizeof("<gml:MultiGeometry></gml:MultiGeometry>");
+
+	if ( srs ) size += strlen(srs) + sizeof(" srsName=..");
+
+	for (i=0; i<insp->ngeometries; i++)
+	{
+		LWPOINT *point;
+		LWLINE *line;
+		LWPOLY *poly;
+		LWGEOM_INSPECTED *subinsp;
+		uchar *subgeom;
+
+		size += sizeof("<gml:geometryMember>/") * 2;
+		if ((point=lwgeom_getpoint_inspected(insp, i)))
+		{
+			size += asgml2_point_size(point, 0, precision);
+			lwpoint_release(point);
+		}
+		else if ((line=lwgeom_getline_inspected(insp, i)))
+		{
+			size += asgml2_line_size(line, 0, precision);
+			lwline_release(line);
+		}
+		else if ((poly=lwgeom_getpoly_inspected(insp, i)))
+		{
+			size += asgml2_poly_size(poly, 0, precision);
+			lwpoly_release(poly);
+		}
+		else
+		{
+			subgeom = lwgeom_getsubgeometry_inspected(insp, i);
+			subinsp = lwgeom_inspect(subgeom);
+			size += asgml2_collection_size(subinsp, 0, precision);
+			lwinspected_release(subinsp);
+		}
+	}
+
+	return size;
+}
+
+/*
+ * Don't call this with single-geoms inspected!
+ */
+static size_t
+asgml2_collection_buf(LWGEOM_INSPECTED *insp, char *srs, char *output, int precision)
+{
+	char *ptr;
+	int i;
+
+	ptr = output;
+
+	/* Open outmost tag */
+	if ( srs )
+	{
+		ptr += sprintf(ptr, "<gml:MultiGeometry srsName=\"%s\">", srs);
+	}
+	else
+	{
+		ptr += sprintf(ptr, "<gml:MultiGeometry>");
+	}
+
+	for (i=0; i<insp->ngeometries; i++)
+	{
+		LWPOINT *point;
+		LWLINE *line;
+		LWPOLY *poly;
+		LWGEOM_INSPECTED *subinsp;
+		uchar *subgeom;
+
+		ptr += sprintf(ptr, "<gml:geometryMember>");
+		if ((point=lwgeom_getpoint_inspected(insp, i)))
+		{
+			ptr += asgml2_point_buf(point, 0, ptr, precision);
+			lwpoint_release(point);
+		}
+		else if ((line=lwgeom_getline_inspected(insp, i)))
+		{
+			ptr += asgml2_line_buf(line, 0, ptr, precision);
+			lwline_release(line);
+		}
+		else if ((poly=lwgeom_getpoly_inspected(insp, i)))
+		{
+			ptr += asgml2_poly_buf(poly, 0, ptr, precision);
+			lwpoly_release(poly);
+		}
+		else
+		{
+			subgeom = lwgeom_getsubgeometry_inspected(insp, i);
+			subinsp = lwgeom_inspect(subgeom);
+			if (lwgeom_getType(subgeom[0]) == COLLECTIONTYPE)
+				ptr += asgml2_collection_buf(subinsp, 0, ptr, precision);
+			else
+				ptr += asgml2_multi_buf(subinsp, 0, ptr, precision);
+			lwinspected_release(subinsp);
+		}
+		ptr += sprintf(ptr, "</gml:geometryMember>");
+	}
+
+	/* Close outmost tag */
+	ptr += sprintf(ptr, "</gml:MultiGeometry>");
+
+	return (ptr-output);
+}
+
+/*
+ * Don't call this with single-geoms inspected!
+ */
+static char *
+asgml2_collection(LWGEOM_INSPECTED *insp, char *srs, int precision)
+{
+	char *gml;
+	size_t size;
+
+	size = asgml2_collection_size(insp, srs, precision);
+	gml = palloc(size);
+	asgml2_collection_buf(insp, srs, gml, precision);
+	return gml;
+}
+
+
 static size_t
 pointArray_toGML2(POINTARRAY *pa, char *output, int precision)
 {
@@ -468,10 +575,19 @@ pointArray_toGML2(POINTARRAY *pa, char *output, int precision)
 		{
 			POINT2D pt;
 			getPoint2d_p(pa, i, &pt);
-			sprintf(x, "%.*f", precision, pt.x);
+
+			if (fabs(pt.x) < MAX_DOUBLE)
+				sprintf(x, "%.*f", precision, pt.x);
+			else
+				sprintf(x, "%g", pt.x);
 			trim_trailing_zeros(x);
-			sprintf(y, "%.*f", precision, pt.y);
+
+			if (fabs(pt.y) < MAX_DOUBLE)
+				sprintf(y, "%.*f", precision, pt.y);
+			else
+				sprintf(y, "%g", pt.y);
 			trim_trailing_zeros(y);
+
 			if ( i ) ptr += sprintf(ptr, " ");
 			ptr += sprintf(ptr, "%s,%s", x, y);
 		}
@@ -482,12 +598,25 @@ pointArray_toGML2(POINTARRAY *pa, char *output, int precision)
 		{
 			POINT4D pt;
 			getPoint4d_p(pa, i, &pt);
-			sprintf(x, "%.*f", precision, pt.x);
+
+			if (fabs(pt.x) < MAX_DOUBLE)
+				sprintf(x, "%.*f", precision, pt.x);
+			else
+				sprintf(x, "%g", pt.x);
 			trim_trailing_zeros(x);
-			sprintf(y, "%.*f", precision, pt.y);
+
+			if (fabs(pt.y) < MAX_DOUBLE)
+				sprintf(y, "%.*f", precision, pt.y);
+			else
+				sprintf(y, "%g", pt.y);
 			trim_trailing_zeros(y);
-			sprintf(z, "%.*f", precision, pt.z);
+
+			if (fabs(pt.z) < MAX_DOUBLE)
+				sprintf(z, "%.*f", precision, pt.z);
+			else
+				sprintf(z, "%g", pt.z);
 			trim_trailing_zeros(z);
+
 			if ( i ) ptr += sprintf(ptr, " ");
 			ptr += sprintf(ptr, "%s,%s,%s", x, y, z);
 		}
@@ -528,9 +657,19 @@ geometry_to_gml3(uchar *geom, char *srs, int precision, bool is_deegree)
 		poly = lwpoly_deserialize(geom);
 		return asgml3_poly(poly, srs, precision, is_deegree);
 
-	default:
+	case MULTIPOINTTYPE:
+	case MULTILINETYPE:
+	case MULTIPOLYGONTYPE:
 		inspected = lwgeom_inspect(geom);
-		return asgml3_inspected(inspected, srs, precision, is_deegree);
+		return asgml3_multi(inspected, srs, precision, is_deegree);
+
+	case COLLECTIONTYPE:
+		inspected = lwgeom_inspect(geom);
+		return asgml3_collection(inspected, srs, precision, is_deegree);
+
+	default:
+		lwerror("geometry_to_gml3: '%s' geometry type not supported", lwgeom_typename(type));
+		return NULL;
 	}
 }
 
@@ -539,7 +678,7 @@ asgml3_point_size(LWPOINT *point, char *srs, int precision)
 {
 	int size;
 	size = pointArray_GMLsize(point->point, precision);
-	size += sizeof("<gml:point><gml:pos>/") * 2;
+	size += sizeof("<gml:point><gml:pos srsDimension='x'>/") * 2;
 	if ( srs ) size += strlen(srs) + sizeof(" srsName=..");
 	return size;
 }
@@ -548,7 +687,9 @@ static size_t
 asgml3_point_buf(LWPOINT *point, char *srs, char *output, int precision, bool is_deegree)
 {
 	char *ptr = output;
+	int dimension=2;
 
+	if (TYPE_HASZ(point->type)) dimension = 3;
 	if ( srs )
 	{
 		ptr += sprintf(ptr, "<gml:Point srsName=\"%s\">", srs);
@@ -557,7 +698,7 @@ asgml3_point_buf(LWPOINT *point, char *srs, char *output, int precision, bool is
 	{
 		ptr += sprintf(ptr, "<gml:Point>");
 	}
-	ptr += sprintf(ptr, "<gml:pos>");
+	ptr += sprintf(ptr, "<gml:pos srsDimension=\"%d\">", dimension);
 	ptr += pointArray_toGML3(point->point, ptr, precision, is_deegree);
 	ptr += sprintf(ptr, "</gml:pos></gml:Point>");
 
@@ -583,6 +724,7 @@ asgml3_line_size(LWLINE *line, char *srs, int precision)
 	int size;
 	size = pointArray_GMLsize(line->points, precision);
 	size += sizeof("<gml:Curve><gml:segments><gml:LineStringSegment><gml:posList>/") * 2;
+	size += sizeof(" srsDimension='x'");
 	if ( srs ) size += strlen(srs) + sizeof(" srsName=..");
 	return size;
 }
@@ -591,7 +733,9 @@ static size_t
 asgml3_line_buf(LWLINE *line, char *srs, char *output, int precision, bool is_deegree)
 {
 	char *ptr=output;
+	int dimension=2;
 
+	if (TYPE_HASZ(line->type)) dimension = 3;
 	if ( srs )
 	{
 		ptr += sprintf(ptr, "<gml:Curve srsName=\"%s\">", srs);
@@ -602,7 +746,7 @@ asgml3_line_buf(LWLINE *line, char *srs, char *output, int precision, bool is_de
 	}
 	ptr += sprintf(ptr, "<gml:segments>");
 	ptr += sprintf(ptr, "<gml:LineStringSegment>");
-	ptr += sprintf(ptr, "<gml:posList>");
+	ptr += sprintf(ptr, "<gml:posList srsDimension=\"%d\">", dimension);
 	ptr += pointArray_toGML3(line->points, ptr, precision, is_deegree);
 	ptr += sprintf(ptr, "</gml:posList></gml:LineStringSegment>");
 	ptr += sprintf(ptr, "</gml:segments>");
@@ -630,16 +774,9 @@ asgml3_poly_size(LWPOLY *poly, char *srs, int precision)
 	size_t size;
 	int i;
 
-	size = sizeof("<gml:Polygon>");
-
-	size += sizeof("<gml:exterior><gml:LinearRing><gml:posList>");
-	size += sizeof("</gml:posList></gml:LinearRing></gml:exterior>");
-
-	size += sizeof("<gml:interior><gml:LinearRing><gml:posList>") * (poly->nrings - 1);
-	size += sizeof("</gml:posList></gml:LinearRing></gml:interior>") * (poly->nrings - 1);
-
-	size += sizeof("</gml:Polygon>");
-
+	size = sizeof("<gml:Polygon><gml:exterior><gml:LinearRing>///") * 2;
+	size += sizeof("<gml:interior><gml:LinearRing>//") * 2 * (poly->nrings - 1);
+	size += sizeof("<gml:posList srsDimension='x'></gml:posList>") * poly->nrings; 
 	if ( srs ) size += strlen(srs) + sizeof(" srsName=..");
 
 	for (i=0; i<poly->nrings; i++)
@@ -653,7 +790,9 @@ asgml3_poly_buf(LWPOLY *poly, char *srs, char *output, int precision, bool is_de
 {
 	int i;
 	char *ptr=output;
+	int dimension=2;
 
+	if (TYPE_HASZ(poly->type)) dimension = 3;
 	if ( srs )
 	{
 		ptr += sprintf(ptr, "<gml:Polygon srsName=\"%s\">", srs);
@@ -662,12 +801,14 @@ asgml3_poly_buf(LWPOLY *poly, char *srs, char *output, int precision, bool is_de
 	{
 		ptr += sprintf(ptr, "<gml:Polygon>");
 	}
-	ptr += sprintf(ptr, "<gml:exterior><gml:LinearRing><gml:posList>");
+	ptr += sprintf(ptr, "<gml:exterior><gml:LinearRing>");
+	ptr += sprintf(ptr, "<gml:posList srsDimension=\"%d\">", dimension);
 	ptr += pointArray_toGML3(poly->rings[0], ptr, precision, is_deegree);
 	ptr += sprintf(ptr, "</gml:posList></gml:LinearRing></gml:exterior>");
 	for (i=1; i<poly->nrings; i++)
 	{
-		ptr += sprintf(ptr, "<gml:interior><gml:LinearRing><gml:posList>");
+		ptr += sprintf(ptr, "<gml:interior><gml:LinearRing>");
+		ptr += sprintf(ptr, "<gml:posList srsDimension=\"%d\">", dimension);
 		ptr += pointArray_toGML3(poly->rings[i], ptr, precision, is_deegree);
 		ptr += sprintf(ptr, "</gml:posList></gml:LinearRing></gml:interior>");
 	}
@@ -688,13 +829,14 @@ asgml3_poly(LWPOLY *poly, char *srs, int precision, bool is_deegree)
 	return output;
 }
 
+
 /*
  * Compute max size required for GML version of this
  * inspected geometry. Will recurse when needed.
  * Don't call this with single-geoms inspected.
  */
 static size_t
-asgml3_inspected_size(LWGEOM_INSPECTED *insp, char *srs, int precision)
+asgml3_multi_size(LWGEOM_INSPECTED *insp, char *srs, int precision)
 {
 	int i;
 	size_t size;
@@ -709,8 +851,6 @@ asgml3_inspected_size(LWGEOM_INSPECTED *insp, char *srs, int precision)
 		LWPOINT *point;
 		LWLINE *line;
 		LWPOLY *poly;
-		LWGEOM_INSPECTED *subinsp;
-		uchar *subgeom;
 
 		if ((point=lwgeom_getpoint_inspected(insp, i)))
 		{
@@ -730,13 +870,6 @@ asgml3_inspected_size(LWGEOM_INSPECTED *insp, char *srs, int precision)
 			size += asgml3_poly_size(poly, 0, precision);
 			lwpoly_release(poly);
 		}
-		else
-		{
-			subgeom = lwgeom_getsubgeometry_inspected(insp, i);
-			subinsp = lwgeom_inspect(subgeom);
-			size += asgml3_inspected_size(subinsp, 0, precision);
-			lwinspected_release(subinsp);
-		}
 	}
 
 	return size;
@@ -746,18 +879,18 @@ asgml3_inspected_size(LWGEOM_INSPECTED *insp, char *srs, int precision)
  * Don't call this with single-geoms inspected!
  */
 static size_t
-asgml3_inspected_buf(LWGEOM_INSPECTED *insp, char *srs, char *output, int precision, bool is_deegree)
+asgml3_multi_buf(LWGEOM_INSPECTED *insp, char *srs, char *output, int precision, bool is_deegree)
 {
 	int type = lwgeom_getType(insp->serialized_form[0]);
 	char *ptr, *gmltype;
 	int i;
 
 	ptr = output;
+	gmltype="";
 
-	if (type == MULTIPOINTTYPE) gmltype = "MultiPoint";
-	else if (type == MULTILINETYPE) gmltype = "MultiCurve";
+	if 	(type == MULTIPOINTTYPE)   gmltype = "MultiPoint";
+	else if (type == MULTILINETYPE)    gmltype = "MultiCurve";
 	else if (type == MULTIPOLYGONTYPE) gmltype = "MultiSurface";
-	else gmltype = "MultiGeometry";
 
 	/* Open outmost tag */
 	if ( srs )
@@ -774,8 +907,6 @@ asgml3_inspected_buf(LWGEOM_INSPECTED *insp, char *srs, char *output, int precis
 		LWPOINT *point;
 		LWLINE *line;
 		LWPOLY *poly;
-		LWGEOM_INSPECTED *subinsp;
-		uchar *subgeom;
 
 		if ((point=lwgeom_getpoint_inspected(insp, i)))
 		{
@@ -798,13 +929,6 @@ asgml3_inspected_buf(LWGEOM_INSPECTED *insp, char *srs, char *output, int precis
 			lwpoly_release(poly);
 			ptr += sprintf(ptr, "</gml:surfaceMember>");
 		}
-		else
-		{
-			subgeom = lwgeom_getsubgeometry_inspected(insp, i);
-			subinsp = lwgeom_inspect(subgeom);
-			ptr += asgml3_inspected_buf(subinsp, 0, ptr, precision, is_deegree);
-			lwinspected_release(subinsp);
-		}
 	}
 
 	/* Close outmost tag */
@@ -817,17 +941,141 @@ asgml3_inspected_buf(LWGEOM_INSPECTED *insp, char *srs, char *output, int precis
  * Don't call this with single-geoms inspected!
  */
 static char *
-asgml3_inspected(LWGEOM_INSPECTED *insp, char *srs, int precision, bool is_deegree)
+asgml3_multi(LWGEOM_INSPECTED *insp, char *srs, int precision, bool is_deegree)
 {
 	char *gml;
 	size_t size;
 
-	size = asgml3_inspected_size(insp, srs, precision);
+	size = asgml3_multi_size(insp, srs, precision);
 	gml = palloc(size);
-	asgml3_inspected_buf(insp, srs, gml, precision, is_deegree);
+	asgml3_multi_buf(insp, srs, gml, precision, is_deegree);
 	return gml;
 }
 
+
+static size_t
+asgml3_collection_size(LWGEOM_INSPECTED *insp, char *srs, int precision)
+{
+	int i;
+	size_t size;
+
+	size = sizeof("<gml:MultiGeometry></gml:MultiGeometry>");
+
+	if ( srs ) size += strlen(srs) + sizeof(" srsName=..");
+
+	for (i=0; i<insp->ngeometries; i++)
+	{
+		LWPOINT *point;
+		LWLINE *line;
+		LWPOLY *poly;
+		LWGEOM_INSPECTED *subinsp;
+		uchar *subgeom;
+
+		size += sizeof("<gml:geometryMember>/") * 2;
+		if ((point=lwgeom_getpoint_inspected(insp, i)))
+		{
+			size += asgml3_point_size(point, 0, precision);
+			lwpoint_release(point);
+		}
+		else if ((line=lwgeom_getline_inspected(insp, i)))
+		{
+			size += asgml3_line_size(line, 0, precision);
+			lwline_release(line);
+		}
+		else if ((poly=lwgeom_getpoly_inspected(insp, i)))
+		{
+			size += asgml3_poly_size(poly, 0, precision);
+			lwpoly_release(poly);
+		}
+		else
+		{
+			subgeom = lwgeom_getsubgeometry_inspected(insp, i);
+			subinsp = lwgeom_inspect(subgeom);
+			size += asgml3_multi_size(subinsp, 0, precision);
+			lwinspected_release(subinsp);
+		}
+	}
+
+	return size;
+}
+
+static size_t
+asgml3_collection_buf(LWGEOM_INSPECTED *insp, char *srs, char *output, int precision, bool is_deegree)
+{
+	char *ptr;
+	int i;
+
+	ptr = output;
+
+	/* Open outmost tag */
+	if ( srs )
+	{
+		ptr += sprintf(ptr, "<gml:MultiGeometry srsName=\"%s\">", srs);
+	}
+	else
+	{
+		ptr += sprintf(ptr, "<gml:MultiGeometry>");
+	}
+
+	for (i=0; i<insp->ngeometries; i++)
+	{
+		LWPOINT *point;
+		LWLINE *line;
+		LWPOLY *poly;
+		LWGEOM_INSPECTED *subinsp;
+		uchar *subgeom;
+
+		ptr += sprintf(ptr, "<gml:geometryMember>");
+		if ((point=lwgeom_getpoint_inspected(insp, i)))
+		{
+			ptr += asgml3_point_buf(point, 0, ptr, precision, is_deegree);
+			lwpoint_release(point);
+		}
+		else if ((line=lwgeom_getline_inspected(insp, i)))
+		{
+			ptr += asgml3_line_buf(line, 0, ptr, precision, is_deegree);
+			lwline_release(line);
+		}
+		else if ((poly=lwgeom_getpoly_inspected(insp, i)))
+		{
+			ptr += asgml3_poly_buf(poly, 0, ptr, precision, is_deegree);
+			lwpoly_release(poly);
+		}
+		else
+		{
+			subgeom = lwgeom_getsubgeometry_inspected(insp, i);
+			subinsp = lwgeom_inspect(subgeom);
+			if (lwgeom_getType(subgeom[0]) == COLLECTIONTYPE)
+				ptr += asgml3_collection_buf(subinsp, 0, ptr, precision, is_deegree);
+			else
+				ptr += asgml3_multi_buf(subinsp, 0, ptr, precision, is_deegree);
+			lwinspected_release(subinsp);
+		}
+		ptr += sprintf(ptr, "</gml:geometryMember>");
+	}
+
+	/* Close outmost tag */
+	ptr += sprintf(ptr, "</gml:MultiGeometry>");
+
+	return (ptr-output);
+}
+
+/*
+ * Don't call this with single-geoms inspected!
+ */
+static char *
+asgml3_collection(LWGEOM_INSPECTED *insp, char *srs, int precision, bool is_deegree)
+{
+	char *gml;
+	size_t size;
+
+	size = asgml3_collection_size(insp, srs, precision);
+	gml = palloc(size);
+	asgml3_collection_buf(insp, srs, gml, precision, is_deegree);
+	return gml;
+}
+
+
 /* In GML3, inside <posList> or <pos>, coordinates are separated by a space separator
  * In GML3 also, lat/lon are reversed for geocentric data
  */
@@ -848,10 +1096,19 @@ pointArray_toGML3(POINTARRAY *pa, char *output, int precision, bool is_deegree)
 		{
 			POINT2D pt;
 			getPoint2d_p(pa, i, &pt);
-			sprintf(x, "%.*f", precision, pt.x);
+
+			if (fabs(pt.x) < MAX_DOUBLE)
+				sprintf(x, "%.*f", precision, pt.x);
+			else
+				sprintf(x, "%g", pt.x);
 			trim_trailing_zeros(x);
-			sprintf(y, "%.*f", precision, pt.y);
+
+			if (fabs(pt.y) < MAX_DOUBLE)
+				sprintf(y, "%.*f", precision, pt.y);
+			else
+				sprintf(y, "%g", pt.y);
 			trim_trailing_zeros(y);
+
 			if ( i ) ptr += sprintf(ptr, " ");
 			if (is_deegree)
 				ptr += sprintf(ptr, "%s %s", y, x);
@@ -865,12 +1122,25 @@ pointArray_toGML3(POINTARRAY *pa, char *output, int precision, bool is_deegree)
 		{
 			POINT4D pt;
 			getPoint4d_p(pa, i, &pt);
-			sprintf(x, "%.*f", precision, pt.x);
+
+			if (fabs(pt.x) < MAX_DOUBLE)
+				sprintf(x, "%.*f", precision, pt.x);
+			else
+				sprintf(x, "%g", pt.x);
 			trim_trailing_zeros(x);
-			sprintf(y, "%.*f", precision, pt.y);
+
+			if (fabs(pt.y) < MAX_DOUBLE)
+				sprintf(y, "%.*f", precision, pt.y);
+			else
+				sprintf(y, "%g", pt.y);
 			trim_trailing_zeros(y);
-			sprintf(z, "%.*f", precision, pt.z);
+
+			if (fabs(pt.z) < MAX_DOUBLE)
+				sprintf(z, "%.*f", precision, pt.z);
+			else
+				sprintf(z, "%g", pt.z);
 			trim_trailing_zeros(z);
+
 			if ( i ) ptr += sprintf(ptr, " ");
 			if (is_deegree)
 				ptr += sprintf(ptr, "%s %s %s", y, x, z);
@@ -884,72 +1154,6 @@ pointArray_toGML3(POINTARRAY *pa, char *output, int precision, bool is_deegree)
 
 
 
-/*
- * Common GML routines
- */
-
-static char *
-getSRSbySRID(int SRID, bool short_crs)
-{
-	char query[128];
-	char *srs, *srscopy;
-	int size, err;
-
-	/* connect to SPI */
-	if (SPI_OK_CONNECT != SPI_connect ())
-	{
-		elog(NOTICE, "getSRSbySRID: could not connect to SPI manager");
-		SPI_finish();
-		return NULL;
-	}
-
-	if (short_crs)
-		sprintf(query, "SELECT auth_name||':'||auth_srid \
-		        FROM spatial_ref_sys WHERE srid='%d'", SRID);
-	else
-		sprintf(query, "SELECT 'urn:ogc:def:crs:'||auth_name||':'||auth_srid \
-		        FROM spatial_ref_sys WHERE srid='%d'", SRID);
-
-	/* execute query */
-	err = SPI_exec(query, 1);
-	if ( err < 0 )
-	{
-		elog(NOTICE, "getSRSbySRID: error executing query %d", err);
-		SPI_finish();
-		return NULL;
-	}
-
-	/* no entry in spatial_ref_sys */
-	if (SPI_processed <= 0)
-	{
-		/*elog(NOTICE, "getSRSbySRID: no record for SRID %d", SRID); */
-		SPI_finish();
-		return NULL;
-	}
-
-	/* get result  */
-	srs = SPI_getvalue(SPI_tuptable->vals[0],
-	                   SPI_tuptable->tupdesc, 1);
-
-	/* NULL result */
-	if ( ! srs )
-	{
-		/*elog(NOTICE, "getSRSbySRID: null result"); */
-		SPI_finish();
-		return NULL;
-	}
-
-	/* copy result to upper executor context */
-	size = strlen(srs)+1;
-	srscopy = SPI_palloc(size);
-	memcpy(srscopy, srs, size);
-
-	/* disconnect from SPI */
-	SPI_finish();
-
-	return srscopy;
-}
-
 /*
  * Returns maximum size of rendered pointarray in bytes.
  */
