@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: lwgeom_ogc.c 4245 2009-07-02 08:08:33Z mleslie $
+ * $Id: lwgeom_ogc.c 5216 2010-02-08 19:19:23Z strk $
  *
  * PostGIS - Spatial Types for PostgreSQL
  * http://postgis.refractions.net
@@ -34,6 +34,7 @@ Datum LWGEOM_getSRID(PG_FUNCTION_ARGS);
 Datum LWGEOM_setSRID(PG_FUNCTION_ARGS);
 /* ---- GeometryType(geometry) */
 Datum LWGEOM_getTYPE(PG_FUNCTION_ARGS);
+Datum geometry_geometrytype(PG_FUNCTION_ARGS);
 /* ---- NumPoints(geometry) */
 Datum LWGEOM_numpoints_linestring(PG_FUNCTION_ARGS);
 /* ---- NumGeometries(geometry) */
@@ -164,11 +165,40 @@ Datum LWGEOM_getTYPE(PG_FUNCTION_ARGS)
 	PG_RETURN_POINTER(text_ob);
 }
 
+
+/* returns a string representation of this geometry's type */
+PG_FUNCTION_INFO_V1(geometry_geometrytype);
+Datum geometry_geometrytype(PG_FUNCTION_ARGS)
+{
+	PG_LWGEOM *lwgeom;
+	char *type_text;
+	char *type_str = palloc(32);
+	size_t size;
+
+	lwgeom = (PG_LWGEOM*)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+
+	/* Make it empty string to start */
+	*type_str = 0;
+
+	/* Build up the output string */
+	strncat(type_str, "ST_", 32);
+	strncat(type_str, lwgeom_typename(lwgeom_getType(lwgeom->type)), 32);
+	size = strlen(type_str) + VARHDRSZ;
+
+	/* Build a text type to store things in */
+	type_text = lwalloc(size);
+	memcpy(VARDATA(type_text),type_str, size - VARHDRSZ);
+	pfree(type_str);
+	SET_VARSIZE(type_text, size);
+	PG_FREE_IF_COPY(lwgeom, 0);
+	PG_RETURN_POINTER(type_text);
+}
+
+
 /**
  * Find first linestring in serialized geometry and return
  * the number of points in it. If no linestrings are found
- * return -1.  Has been expanded to handle circular strings
- * as well.
+ * return -1.
  */
 static int32
 lwgeom_numpoints_linestring_recursive(const uchar *serialized)
@@ -178,12 +208,12 @@ lwgeom_numpoints_linestring_recursive(const uchar *serialized)
 
 	LWDEBUG(2, "lwgeom_numpoints_linestring_recursive called.");
 
-	/* 
+	/*
 	 * CURVEPOLY and COMPOUND have no concept of numpoints but look like
 	 * collections once inspected.  Fast-fail on these here.
 	 */
 	if (lwgeom_getType(inspected->type) == COMPOUNDTYPE ||
-			lwgeom_getType(inspected->type) == CURVEPOLYTYPE)
+	        lwgeom_getType(inspected->type) == CURVEPOLYTYPE)
 	{
 		return -1;
 	}
@@ -205,7 +235,7 @@ lwgeom_numpoints_linestring_recursive(const uchar *serialized)
 		}
 		else if (lwgeom_getType(geom->type) == CIRCSTRINGTYPE)
 		{
-			return((LWCIRCSTRING *)geom)->points->npoints;
+			return ((LWCIRCSTRING *)geom)->points->npoints;
 		}
 
 		subgeom = lwgeom_getsubgeometry_inspected(inspected, i);
@@ -216,10 +246,8 @@ lwgeom_numpoints_linestring_recursive(const uchar *serialized)
 
 		type = lwgeom_getType(subgeom[0]);
 
-		/* MULTILINESTRING && MULTICURVE && GEOMETRYCOLLECTION are worth checking */
-		if ( type != MULTILINETYPE && 
-			 type != COLLECTIONTYPE &&
-			 type != MULTICURVETYPE ) continue;
+		/* MULTILINESTRING && GEOMETRYCOLLECTION are worth checking */
+		if ( type != MULTILINETYPE && type != COLLECTIONTYPE ) continue;
 
 		npoints = lwgeom_numpoints_linestring_recursive(subgeom);
 		if ( npoints == -1 ) continue;
@@ -1008,18 +1036,6 @@ Datum LWGEOM_from_text(PG_FUNCTION_ARGS)
 
 	fc=*(VARDATA(wkttext));
 
-	/*
-	 * 'S' is accepted for backward compatibility, a WARNING
-	 * is issued later.
-	 *
-	 * TODO: reject 'SRID=x;..' form...
-	 */
-	if ( fc!='P' && fc!='L' && fc!='M' && fc!='G' && fc!='S' && fc!='C' )
-	{
-		lwerror("Invalid OGC WKT (does not start with P,L,M,C or G)");
-		PG_RETURN_NULL();
-	}
-
 	wkt = lwalloc(size+1);
 	memcpy(wkt, VARDATA(wkttext), size);
 	wkt[size]='\0';
@@ -1105,7 +1121,7 @@ Datum LWGEOM_asText(PG_FUNCTION_ARGS)
 	ogclwgeom = (PG_LWGEOM *)DatumGetPointer(DirectFunctionCall1(
 	                LWGEOM_force_2d, PointerGetDatum(lwgeom)));
 
-	result =  serialized_lwgeom_to_ewkt(&lwg_unparser_result, SERIALIZED_FORM(ogclwgeom), PARSER_CHECK_ALL);
+	result =  serialized_lwgeom_to_ewkt(&lwg_unparser_result, SERIALIZED_FORM(ogclwgeom), PARSER_CHECK_NONE);
 	if (result)
 		PG_UNPARSER_ERROR(lwg_unparser_result);
 
