@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: lwgeom_rtree.c 4211 2009-06-25 03:32:43Z robe $
+ * $Id: lwgeom_rtree.c 7140 2011-05-13 01:01:26Z chodgson $
  *
  * PostGIS - Spatial Types for PostgreSQL
  * http://postgis.refractions.net
@@ -218,17 +218,23 @@ void freeTree(RTREE_NODE *root)
  */
 void clearCache(RTREE_POLY_CACHE *cache)
 {
-	int i;
+	int g, r, i;
 	LWDEBUGF(2, "clearCache called for %p", cache);
-	for (i = 0; i < cache->ringCount; i++)
-	{
-		freeTree(cache->ringIndices[i]);
+        i = 0;
+        for (g = 0; g < cache->polyCount; g++)
+        {
+	        for (r = 0; r < cache->ringCounts[g]; r++)
+	        {
+		        freeTree(cache->ringIndices[i]);
+                        i++;
+                }
 	}
 	lwfree(cache->ringIndices);
+        lwfree(cache->ringCounts);
 	lwfree(cache->poly);
 	cache->poly = 0;
 	cache->ringIndices = 0;
-	cache->ringCount = 0;
+	cache->ringCounts = 0;
 	cache->polyCount = 0;
 }
 
@@ -397,7 +403,7 @@ RTREE_POLY_CACHE * createCache()
 	RTREE_POLY_CACHE *result;
 	result = lwalloc(sizeof(RTREE_POLY_CACHE));
 	result->polyCount = 0;
-	result->ringCount = 0;
+	result->ringCounts = 0;
 	result->ringIndices = 0;
 	result->poly = 0;
 	result->type = 1;
@@ -406,7 +412,7 @@ RTREE_POLY_CACHE * createCache()
 
 void populateCache(RTREE_POLY_CACHE *currentCache, LWGEOM *lwgeom, uchar *serializedPoly)
 {
-	int i, j, k, length;
+	int i, p, r, length;
 	LWMPOLY *mpoly;
 	LWPOLY *poly;
 	int nrings;
@@ -421,28 +427,24 @@ void populateCache(RTREE_POLY_CACHE *currentCache, LWGEOM *lwgeom, uchar *serial
 		/*
 		** Count the total number of rings.
 		*/
+		currentCache->polyCount = mpoly->ngeoms;
+		currentCache->ringCounts = lwalloc(sizeof(int) * mpoly->ngeoms);
 		for ( i = 0; i < mpoly->ngeoms; i++ )
 		{
+                        currentCache->ringCounts[i] = mpoly->geoms[i]->nrings;
 			nrings += mpoly->geoms[i]->nrings;
 		}
-		currentCache->polyCount = mpoly->ngeoms;
-		currentCache->ringCount = nrings;
 		currentCache->ringIndices = lwalloc(sizeof(RTREE_NODE *) * nrings);
 		/*
-		** Load the exterior rings onto the ringIndices array first
+		** Load the array in geometry order, each outer ring followed by the inner rings
+                ** associated with that outer ring
 		*/
-		for ( i = 0; i < mpoly->ngeoms; i++ )
+                i = 0;
+		for ( p = 0; p < mpoly->ngeoms; p++ )
 		{
-			currentCache->ringIndices[i] = createTree(mpoly->geoms[i]->rings[0]);
-		}
-		/*
-		** Load the interior rings (holes) onto ringIndices next
-		*/
-		for ( j = 0; j < mpoly->ngeoms; j++ )
-		{
-			for ( k = 1; k < mpoly->geoms[j]->nrings; k++ )
+			for ( r = 0; r < mpoly->geoms[p]->nrings; r++ )
 			{
-				currentCache->ringIndices[i] = createTree(mpoly->geoms[j]->rings[k]);
+				currentCache->ringIndices[i] = createTree(mpoly->geoms[p]->rings[r]);
 				i++;
 			}
 		}
@@ -452,7 +454,8 @@ void populateCache(RTREE_POLY_CACHE *currentCache, LWGEOM *lwgeom, uchar *serial
 		LWDEBUG(2, "populateCache POLYGON");
 		poly = (LWPOLY *)lwgeom;
 		currentCache->polyCount = 1;
-		currentCache->ringCount = poly->nrings;
+		currentCache->ringCounts = lwalloc(sizeof(int));
+		currentCache->ringCounts[0] = poly->nrings;
 		/*
 		** Just load the rings on in order
 		*/
