@@ -1,10 +1,11 @@
 -- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 -- 
--- $Id: uninstall_postgis.sql.in.c 3993 2009-04-12 10:56:33Z mcayland $
+-- $Id: uninstall_postgis.sql.in.c 5166 2010-01-26 17:09:30Z pramsey $
 --
 -- PostGIS - Spatial Types for PostgreSQL
 -- http://postgis.refractions.net
 -- Copyright 2001-2003 Refractions Research Inc.
+-- Copyright 2009 Open Source Geospatial Foundation <http://osgeo.org>
 --
 -- This is free software; you can redistribute and/or modify it under
 -- the terms of the GNU General Public Licence. See the COPYING file.
@@ -26,6 +27,10 @@ BEGIN;
 
 DROP FUNCTION ST_MinimumBoundingCircle(geometry);
 DROP FUNCTION ST_MinimumBoundingCircle(inputgeom geometry, segs_per_quarter integer);
+DROP FUNCTION ST_ShortestLine(geometry, geometry);
+DROP FUNCTION ST_LongestLine(geometry, geometry);
+DROP FUNCTION ST_DFullyWithin(geometry, geometry, float8);
+DROP FUNCTION _ST_DumpPoints(geometry, integer[]);
 
 ---------------------------------------------------------------
 -- SQL-MM
@@ -42,6 +47,7 @@ DROP FUNCTION ST_CurveToLine(geometry, integer);
 
 #include "uninstall_sqlmm.sql.in.c"
 #include "uninstall_long_xact.sql.in.c"
+#include "uninstall_geography.sql.in.c"
 
 DROP FUNCTION ST_BdMPolyFromText(text, integer);
 DROP FUNCTION BdMPolyFromText(text, integer);
@@ -243,6 +249,9 @@ DROP FUNCTION AsGML(geometry);
 DROP FUNCTION ST_AsGML(geometry, int4);
 DROP FUNCTION AsGML(geometry, int4);
 DROP FUNCTION _ST_AsGML(int4, geometry, int4, int4);
+DROP FUNCTION ST_GeomFromGML(text);
+DROP FUNCTION ST_GMLToSQL(text);
+DROP FUNCTION ST_GeomFromKML(text);
 
 -----------------------------------------------------------------------
 -- SVG OUTPUT
@@ -267,24 +276,17 @@ DROP FUNCTION ST_IsRing(geometry);
 DROP FUNCTION IsRing(geometry);
 DROP FUNCTION ST_Centroid(geometry);
 DROP FUNCTION Centroid(geometry);
-DROP FUNCTION GEOSnoop(geometry);
 DROP FUNCTION ST_IsValid(geometry);
 DROP FUNCTION IsValid(geometry);
 DROP FUNCTION ST_Overlaps(geometry,geometry);
 DROP FUNCTION _ST_Overlaps(geometry,geometry);
 DROP FUNCTION overlaps(geometry,geometry);
-
-#if POSTGIS_GEOS_VERSION >= 31
 DROP FUNCTION ST_ContainsProperly(geometry,geometry);
 DROP FUNCTION _ST_ContainsProperly(geometry,geometry);
-#endif
-
-#if POSTGIS_GEOS_VERSION >= 30
 DROP FUNCTION ST_Covers(geometry,geometry);
 DROP FUNCTION _ST_Covers(geometry,geometry);
 DROP FUNCTION ST_CoveredBy(geometry,geometry);
 DROP FUNCTION _ST_CoveredBy(geometry,geometry);
-#endif
 
 DROP FUNCTION ST_Contains(geometry,geometry);
 DROP FUNCTION _ST_Contains(geometry,geometry);
@@ -322,20 +324,10 @@ DROP AGGREGATE ST_Collect(geometry);
 DROP AGGREGATE collect(geometry);
 DROP AGGREGATE ST_Union(geometry);
 
--- TO BE REMOVED BEFORE RELEASE
-DROP AGGREGATE ST_Union_Old(geometry);
--- TO BE REMOVED BEFORE RELEASE
-DROP AGGREGATE GeomUnion_Old(geometry);
-
 DROP FUNCTION ST_Union (geometry[]);
 DROP FUNCTION ST_unite_garray (geometry[]);
 DROP FUNCTION unite_garray (geometry[]);
 
--- TO BE REMOVED BEFORE RELEASE
-DROP AGGREGATE ST_accum_old(geometry);
--- TO BE REMOVED BEFORE RELEASE
-DROP AGGREGATE accum_old(geometry);
-
 DROP AGGREGATE ST_Accum(geometry);
 DROP AGGREGATE accum(geometry);
 
@@ -351,16 +343,10 @@ DROP TYPE pgis_abs CASCADE;
 DROP AGGREGATE ST_MemUnion(geometry);
 DROP AGGREGATE MemGeomUnion(geometry);
 DROP FUNCTION ST_collect(geometry[]);
-DROP FUNCTION ST_collect_garray(geometry[]);
-DROP FUNCTION collect_garray(geometry[]);
-DROP FUNCTION ST_geom_accum(geometry[],geometry);
-DROP FUNCTION geom_accum (geometry[],geometry);
 DROP AGGREGATE ST_memcollect(geometry);
 DROP AGGREGATE memcollect(geometry);
 DROP FUNCTION ST_collect(geometry, geometry);
 DROP FUNCTION collect(geometry, geometry);
-DROP FUNCTION ST_collector(geometry, geometry);
-DROP FUNCTION collector(geometry, geometry);
 
 
 ---------------------------------------------------------------
@@ -377,15 +363,8 @@ DROP FUNCTION ST_Boundary(geometry);
 DROP FUNCTION boundary(geometry);
 DROP FUNCTION ST_Difference(geometry,geometry);
 DROP FUNCTION difference(geometry,geometry);
-
-#if POSTGIS_GEOS_VERSION >= 31
 DROP FUNCTION ST_IsValidReason(geometry);
-#endif
-
-#if POSTGIS_GEOS_VERSION >= 30
 DROP FUNCTION ST_SimplifyPreserveTopology(geometry, float8);
-#endif
-
 DROP FUNCTION ST_LocateBetweenElevations(geometry, float8, float8);
 DROP FUNCTION ST_LineCrossingDirection(geometry, geometry);
 DROP FUNCTION _ST_LineCrossingDirection(geometry, geometry);
@@ -395,8 +374,14 @@ DROP FUNCTION ST_buffer(geometry,float8,integer);
 DROP FUNCTION buffer(geometry,float8,integer);
 DROP FUNCTION ST_Buffer(geometry,float8);
 DROP FUNCTION buffer(geometry,float8);
+DROP FUNCTION ST_buffer(geometry,float8,text);
+DROP FUNCTION _ST_buffer(geometry,float8,cstring);
 DROP FUNCTION ST_Intersection(geometry,geometry);
 DROP FUNCTION intersection(geometry,geometry);
+#if POSTGIS_GEOS_VERSION >= 32
+DROP FUNCTION ST_HausdorffDistance(geometry, geometry);
+DROP FUNCTION ST_HausdorffDistance(geometry, geometry, float8);
+#endif
 
 
 ---------------------------------------------------------------
@@ -413,7 +398,7 @@ DROP FUNCTION ST_line_substring(geometry, float8, float8);
 DROP FUNCTION line_substring(geometry, float8, float8);
 DROP FUNCTION ST_line_interpolate_point(geometry, float8);
 DROP FUNCTION line_interpolate_point(geometry, float8);
-
+DROP FUNCTION ST_AddMeasure(geometry, float8, float8);
 
 ---------------------------------------------------------------
 -- Algorithms
@@ -489,16 +474,16 @@ DROP FUNCTION box2d(geometry);
 -----------------------------------------------------------------------
 
 DROP FUNCTION postgis_full_version();
+DROP FUNCTION postgis_lib_version();
+DROP FUNCTION postgis_version();
 DROP FUNCTION postgis_lib_build_date();
 DROP FUNCTION postgis_scripts_build_date();
 DROP FUNCTION postgis_geos_version();
+DROP FUNCTION postgis_libxml_version();
 DROP FUNCTION postgis_uses_stats();
 DROP FUNCTION postgis_scripts_released();
-DROP FUNCTION postgis_lib_version();
 DROP FUNCTION postgis_scripts_installed();
 DROP FUNCTION postgis_proj_version();
-DROP FUNCTION postgis_version();
-
 
 ---------------------------------------------------------------
 -- PROJ support
@@ -506,7 +491,6 @@ DROP FUNCTION postgis_version();
 
 DROP FUNCTION ST_Transform(geometry,integer);
 DROP FUNCTION transform(geometry,integer);
-DROP FUNCTION transform_geometry(geometry,text,text,int);
 DROP FUNCTION get_proj4_from_srid(integer);
 
 
@@ -576,7 +560,6 @@ DROP FUNCTION ST_DumpRings(geometry);
 DROP FUNCTION DumpRings(geometry);
 DROP FUNCTION ST_Dump(geometry);
 DROP FUNCTION Dump(geometry);
-DROP TYPE geometry_dump;
 DROP FUNCTION ST_LineMerge(geometry);
 DROP FUNCTION LineMerge(geometry);
 DROP FUNCTION ST_Polygonize (geometry[]);
@@ -621,8 +604,6 @@ DROP FUNCTION MakePoint(float8, float8);
 -- MISC
 ------------------------------------------------------------------------
 
-DROP FUNCTION ST_Cache_BBox();
-DROP FUNCTION cache_bbox();
 DROP FUNCTION ST_GeomFromEWKT(text);
 DROP FUNCTION GeomFromEWKT(text);
 DROP FUNCTION ST_GeomFromEWKB(bytea);
@@ -639,11 +620,10 @@ DROP FUNCTION ST_AsEWKT(geometry);
 DROP FUNCTION AsEWKT(geometry);
 DROP FUNCTION ST_NDims(geometry);
 DROP FUNCTION ndims(geometry);
-DROP FUNCTION ST_HasBBOX(geometry);
 DROP FUNCTION hasBBOX(geometry);
+DROP FUNCTION postgis_cache_bbox();
 DROP FUNCTION ST_zmflag(geometry);
 DROP FUNCTION zmflag(geometry);
-DROP FUNCTION ST_noop(geometry);
 DROP FUNCTION noop(geometry);
 DROP FUNCTION ST_ForceRHR(geometry);
 DROP FUNCTION ForceRHR(geometry);
@@ -680,7 +660,8 @@ DROP FUNCTION ST_azimuth(geometry,geometry);
 DROP FUNCTION azimuth(geometry,geometry);
 DROP FUNCTION ST_point_inside_circle(geometry,float8,float8,float8);
 DROP FUNCTION point_inside_circle(geometry,float8,float8,float8);
-DROP FUNCTION ST_max_distance(geometry,geometry);
+DROP FUNCTION _ST_MaxDistance(geometry,geometry);
+DROP FUNCTION ST_maxdistance(geometry,geometry);
 DROP FUNCTION max_distance(geometry,geometry);
 DROP FUNCTION ST_Distance(geometry,geometry);
 DROP FUNCTION distance(geometry,geometry);
@@ -753,9 +734,7 @@ DROP FUNCTION srid(chip);
 
 DROP FUNCTION getBBOX(geometry);
 DROP FUNCTION getSRID(geometry);
-DROP FUNCTION ST_dropBBOX(geometry);
 DROP FUNCTION dropBBOX(geometry);
-DROP FUNCTION ST_addBBOX(geometry);
 DROP FUNCTION addBBOX(geometry);
 
 
@@ -855,27 +834,6 @@ DROP FUNCTION ST_geometry_lt(geometry, geometry);
 DROP FUNCTION geometry_lt(geometry, geometry);
 
 
----- BOX2D  support functions
-
-DROP FUNCTION ST_box2d_intersects(box2d, box2d);
-DROP FUNCTION box2d_intersects(box2d, box2d);
-DROP FUNCTION ST_box2d_same(box2d, box2d);
-DROP FUNCTION box2d_same(box2d, box2d);
-DROP FUNCTION ST_box2d_overlap(box2d, box2d);
-DROP FUNCTION box2d_overlap(box2d, box2d);
-DROP FUNCTION ST_box2d_contained(box2d, box2d);
-DROP FUNCTION box2d_contained(box2d, box2d);
-DROP FUNCTION ST_box2d_contain(box2d, box2d);
-DROP FUNCTION box2d_contain(box2d, box2d);
-DROP FUNCTION ST_box2d_right(box2d, box2d);
-DROP FUNCTION box2d_right(box2d, box2d);
-DROP FUNCTION ST_box2d_left(box2d, box2d);
-DROP FUNCTION box2d_left(box2d, box2d);
-DROP FUNCTION ST_box2d_overright(box2d, box2d);
-DROP FUNCTION box2d_overright(box2d, box2d);
-DROP FUNCTION ST_box2d_overleft(box2d, box2d);
-DROP FUNCTION box2d_overleft(box2d, box2d);
-
 
 -----------------------------------------------------------------------
 -- BOX2D
@@ -953,6 +911,11 @@ DROP FUNCTION ST_Affine(geometry,float8,float8,float8,float8,float8,float8,float
 DROP FUNCTION Affine(geometry,float8,float8,float8,float8,float8,float8,float8,float8,float8,float8,float8,float8);
 
 
+-------------------------------------------------------------------
+--  GEOMETRY TYPE (geometry_dump)
+-------------------------------------------------------------------
+DROP TYPE geometry_dump CASCADE;
+
 -------------------------------------------------------------------
 --  GEOMETRY TYPE (lwgeom)
 -------------------------------------------------------------------
@@ -964,6 +927,8 @@ DROP TYPE geometry CASCADE;
 
 DROP FUNCTION ST_geometry_analyze(internal);
 DROP FUNCTION geometry_analyze(internal);
+DROP FUNCTION geometry_gist_sel (internal, oid, internal, int4);
+DROP FUNCTION geometry_gist_joinsel(internal, oid, internal, smallint);
 
 -------------------------------------------------------------------
 --  SPHEROID TYPE
